@@ -1,25 +1,13 @@
 /*
- *      This file is part of GPaste.
+ * This file is part of GPaste.
  *
- *      Copyright 2011-2015 Marc-Antoine Perennou <Marc-Antoine@Perennou.com>
- *
- *      GPaste is free software: you can redistribute it and/or modify
- *      it under the terms of the GNU General Public License as published by
- *      the Free Software Foundation, either version 3 of the License, or
- *      (at your option) any later version.
- *
- *      GPaste is distributed in the hope that it will be useful,
- *      but WITHOUT ANY WARRANTY; without even the implied warranty of
- *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *      GNU General Public License for more details.
- *
- *      You should have received a copy of the GNU General Public License
- *      along with GPaste.  If not, see <http://www.gnu.org/licenses/>.
+ * Copyright (c) 2010-2018, Marc-Antoine Perennou <Marc-Antoine@Perennou.com>
  */
 
 #include <gpaste-clipboard.h>
 #include <gpaste-image-item.h>
 #include <gpaste-uris-item.h>
+#include <gpaste-util.h>
 
 #include <string.h>
 
@@ -28,18 +16,24 @@ struct _GPasteClipboard
     GObject parent_instance;
 };
 
+enum
+{
+    C_OWNER_CHANGE,
+
+    C_LAST_SIGNAL
+};
+
 typedef struct
 {
-    GdkAtom         target;
     GtkClipboard   *real;
     GPasteSettings *settings;
     gchar          *text;
     gchar          *image_checksum;
 
-    gulong          owner_change_signal;
+    guint64         c_signals[C_LAST_SIGNAL];
 } GPasteClipboardPrivate;
 
-G_DEFINE_TYPE_WITH_PRIVATE (GPasteClipboard, g_paste_clipboard, G_TYPE_OBJECT)
+G_PASTE_DEFINE_TYPE_WITH_PRIVATE (Clipboard, clipboard, G_TYPE_OBJECT)
 
 enum
 {
@@ -48,19 +42,24 @@ enum
     LAST_SIGNAL
 };
 
-static guint signals[LAST_SIGNAL] = { 0 };
+static guint64 signals[LAST_SIGNAL] = { 0 };
 
 static void
 g_paste_clipboard_bootstrap_finish (GPasteClipboard *self,
                                     GPasteHistory   *history)
 {
-    GPasteClipboardPrivate  *priv = g_paste_clipboard_get_instance_private (self);
+    const GPasteClipboardPrivate *priv = _g_paste_clipboard_get_instance_private (self);
 
     if (!priv->text && !priv->image_checksum)
     {
-        const GSList *h = g_paste_history_get_history (history);
+        const GList *h = g_paste_history_get_history (history);
         if (h)
-            g_paste_clipboard_select_item (self, h->data);
+        {
+            GPasteItem *item = h->data;
+
+            if (!g_paste_clipboard_select_item (self, item))
+                g_paste_history_remove (history, 0);
+        }
     }
 }
 
@@ -74,9 +73,10 @@ g_paste_clipboard_bootstrap_finish_text (GPasteClipboard *self,
 
 static void
 g_paste_clipboard_bootstrap_finish_image (GPasteClipboard *self,
-                                          GdkPixbuf       *image G_GNUC_UNUSED,
+                                          GdkPixbuf       *image,
                                           gpointer         user_data)
 {
+    g_object_unref (image);
     g_paste_clipboard_bootstrap_finish (self, user_data);
 }
 
@@ -86,17 +86,15 @@ g_paste_clipboard_bootstrap_finish_image (GPasteClipboard *self,
  * @history: a #GPasteHistory instance
  *
  * Bootstrap a #GPasteClipboard with an initial value
- *
- * Returns:
  */
 G_PASTE_VISIBLE void
 g_paste_clipboard_bootstrap (GPasteClipboard *self,
                              GPasteHistory   *history)
 {
-    g_return_if_fail (G_PASTE_IS_CLIPBOARD (self));
-    g_return_if_fail (G_PASTE_IS_HISTORY (history));
+    g_return_if_fail (_G_PASTE_IS_CLIPBOARD (self));
+    g_return_if_fail (_G_PASTE_IS_HISTORY (history));
 
-    GPasteClipboardPrivate  *priv = g_paste_clipboard_get_instance_private (self);
+    const GPasteClipboardPrivate *priv = _g_paste_clipboard_get_instance_private (self);
     GtkClipboard *real = priv->real;
 
     if (gtk_clipboard_wait_is_uris_available (real) ||
@@ -106,30 +104,34 @@ g_paste_clipboard_bootstrap (GPasteClipboard *self,
                                     g_paste_clipboard_bootstrap_finish_text,
                                     history);
     }
-    else if (gtk_clipboard_wait_is_image_available (real))
+    else if (g_paste_settings_get_images_support (priv->settings) && gtk_clipboard_wait_is_image_available (real))
     {
         g_paste_clipboard_set_image (self,
                                      g_paste_clipboard_bootstrap_finish_image,
                                      history);
     }
+    else
+    {
+        g_paste_clipboard_ensure_not_empty (self, history);
+    }
 }
 
 /**
- * g_paste_clipboard_get_target:
+ * g_paste_clipboard_is_clipboard:
  * @self: a #GPasteClipboard instance
  *
- * Get the target the #GPasteClipboard points to
+ * Get whether this #GPasteClipboard is a clipboard or not (primary selection)
  *
- * Returns: (transfer none): the GdkAtom representing the target (Primary, Clipboard, ...)
+ * Returns: %TRUE if this #GPasteClipboard is a clipboard
  */
-G_PASTE_VISIBLE GdkAtom
-g_paste_clipboard_get_target (const GPasteClipboard *self)
+G_PASTE_VISIBLE gboolean
+g_paste_clipboard_is_clipboard (const GPasteClipboard *self)
 {
-    g_return_val_if_fail (G_PASTE_IS_CLIPBOARD (self), NULL);
+    g_return_val_if_fail (_G_PASTE_IS_CLIPBOARD (self), FALSE);
 
-    GPasteClipboardPrivate *priv = g_paste_clipboard_get_instance_private (self);
+    const GPasteClipboardPrivate *priv = _g_paste_clipboard_get_instance_private (self);
 
-    return priv->target;
+    return gtk_clipboard_get_selection (priv->real) == GDK_SELECTION_CLIPBOARD;
 }
 
 /**
@@ -143,9 +145,9 @@ g_paste_clipboard_get_target (const GPasteClipboard *self)
 G_PASTE_VISIBLE GtkClipboard *
 g_paste_clipboard_get_real (const GPasteClipboard *self)
 {
-    g_return_val_if_fail (G_PASTE_IS_CLIPBOARD (self), NULL);
+    g_return_val_if_fail (_G_PASTE_IS_CLIPBOARD (self), NULL);
 
-    GPasteClipboardPrivate *priv = g_paste_clipboard_get_instance_private (self);
+    const GPasteClipboardPrivate *priv = _g_paste_clipboard_get_instance_private (self);
 
     return priv->real;
 }
@@ -161,13 +163,26 @@ g_paste_clipboard_get_real (const GPasteClipboard *self)
 G_PASTE_VISIBLE const gchar *
 g_paste_clipboard_get_text (const GPasteClipboard *self)
 {
-    g_return_val_if_fail (G_PASTE_IS_CLIPBOARD (self), NULL);
+    g_return_val_if_fail (_G_PASTE_IS_CLIPBOARD (self), NULL);
 
-    GPasteClipboardPrivate *priv = g_paste_clipboard_get_instance_private (self);
+    const GPasteClipboardPrivate *priv = _g_paste_clipboard_get_instance_private (self);
 
     return priv->text;
 }
 
+static const gchar *
+_g_paste_clipboard_private_target_name (const GPasteClipboardPrivate *priv)
+{
+    GdkAtom target = gtk_clipboard_get_selection (priv->real);
+
+    if (target == GDK_SELECTION_CLIPBOARD)
+        return "CLIPBOARD";
+    else if (target == GDK_SELECTION_PRIMARY)
+        return "PRIMARY";
+    else
+        return "UNKNOWN";
+}
+
 static void
 g_paste_clipboard_private_set_text (GPasteClipboardPrivate *priv,
                                     const gchar            *text)
@@ -175,6 +190,8 @@ g_paste_clipboard_private_set_text (GPasteClipboardPrivate *priv,
     g_free (priv->text);
     g_free (priv->image_checksum);
 
+    g_debug("%s: set text", _g_paste_clipboard_private_target_name (priv));
+
     priv->text = g_strdup (text);
     priv->image_checksum = NULL;
 }
@@ -205,7 +222,7 @@ g_paste_clipboard_on_text_ready (GtkClipboard *clipboard G_GNUC_UNUSED,
     g_autofree gchar *stripped = g_strstrip (g_strdup (text));
     gboolean trim_items = g_paste_settings_get_trim_items (settings);
     const gchar *to_add = trim_items ? stripped : text;
-    gsize length = strlen (to_add);
+    guint64 length = strlen (to_add);
 
     if (length < g_paste_settings_get_min_text_item_size (settings) ||
         length > g_paste_settings_get_max_text_item_size (settings) ||
@@ -215,7 +232,7 @@ g_paste_clipboard_on_text_ready (GtkClipboard *clipboard G_GNUC_UNUSED,
             data->callback (self, NULL, data->user_data);
         return;
     }
-    if (priv->text && !g_strcmp0 (priv->text, to_add))
+    if (priv->text && g_paste_str_equal (priv->text, to_add))
     {
         if (data->callback)
             data->callback (self, NULL, data->user_data);
@@ -223,8 +240,8 @@ g_paste_clipboard_on_text_ready (GtkClipboard *clipboard G_GNUC_UNUSED,
     }
 
     if (trim_items &&
-        priv->target == GDK_SELECTION_CLIPBOARD &&
-        g_strcmp0 (text, stripped))
+        gtk_clipboard_get_selection (priv->real) == GDK_SELECTION_CLIPBOARD &&
+        !g_paste_str_equal (text, stripped))
             g_paste_clipboard_select_text (self, stripped);
     else
         g_paste_clipboard_private_set_text (priv, to_add);
@@ -240,17 +257,15 @@ g_paste_clipboard_on_text_ready (GtkClipboard *clipboard G_GNUC_UNUSED,
  * @user_data: user data to pass to @callback
  *
  * Put the text from the intern GtkClipboard in the #GPasteClipboard
- *
- * Returns:
  */
 G_PASTE_VISIBLE void
 g_paste_clipboard_set_text (GPasteClipboard            *self,
                             GPasteClipboardTextCallback callback,
                             gpointer                    user_data)
 {
-    g_return_if_fail (G_PASTE_IS_CLIPBOARD (self));
+    g_return_if_fail (_G_PASTE_IS_CLIPBOARD (self));
 
-    GPasteClipboardPrivate *priv = g_paste_clipboard_get_instance_private (self);
+    const GPasteClipboardPrivate *priv = _g_paste_clipboard_get_instance_private (self);
     GPasteClipboardTextCallbackData *data = g_new (GPasteClipboardTextCallbackData, 1);
 
     data->self = self;
@@ -268,74 +283,130 @@ g_paste_clipboard_set_text (GPasteClipboard            *self,
  * @text: the text to select
  *
  * Put the text into the #GPasteClipbaord and the intern GtkClipboard
- *
- * Returns:
  */
 G_PASTE_VISIBLE void
 g_paste_clipboard_select_text (GPasteClipboard *self,
                                const gchar     *text)
 {
-    g_return_if_fail (G_PASTE_IS_CLIPBOARD (self));
+    g_return_if_fail (_G_PASTE_IS_CLIPBOARD (self));
     g_return_if_fail (text);
     g_return_if_fail (g_utf8_validate (text, -1, NULL));
 
     GPasteClipboardPrivate *priv = g_paste_clipboard_get_instance_private (self);
-    GtkClipboard *real = priv->real;
 
-    /* Avoid cycling twice */
+    g_debug("%s: select text", _g_paste_clipboard_private_target_name (priv));
+
+    /* Avoid cycling twice as gtk_clipboard_set_text will make the clipboards manager react */
     g_paste_clipboard_private_set_text (priv, text);
+    gtk_clipboard_set_text (priv->real, text, -1);
+}
 
-    /* Let the clipboards manager handle our internal text */
-    gtk_clipboard_set_text (real, text, -1);
-    gtk_clipboard_store (real);
+static void
+g_paste_clipboard_sync_ready (GtkClipboard *clipboard G_GNUC_UNUSED,
+                              const gchar  *text,
+                              gpointer      user_data)
+{
+    if (text)
+        g_paste_clipboard_select_text (user_data, text);
+}
+
+/**
+ * g_paste_clipboard_sync_text:
+ * @self: the source #GPasteClipboard instance
+ * @other: the target #GPasteClipboard instance
+ *
+ * Synchronise the text between two clipboards
+ */
+G_PASTE_VISIBLE void
+g_paste_clipboard_sync_text (const GPasteClipboard *self,
+                             GPasteClipboard       *other)
+{
+    g_return_if_fail (_G_PASTE_IS_CLIPBOARD (self));
+    g_return_if_fail (_G_PASTE_IS_CLIPBOARD (other));
+
+    const GPasteClipboardPrivate *priv = _g_paste_clipboard_get_instance_private (self);
+
+    gtk_clipboard_request_text (priv->real, g_paste_clipboard_sync_ready, other);
+}
+
+static guchar *
+copy_str_as_uchars (const gchar *str,
+                    guint64      length)
+{
+    guchar *data = g_new (guchar, length);
+    for (guint64 i = 0; i < length; ++i)
+        data[i] = (guchar) str[i];
+    return data;
+}
+
+static void
+_get_clipboard_data_from_special_atom (GtkSelectionData *selection_data,
+                                       const GPasteItem *item,
+                                       GPasteSpecialAtom atom)
+{
+    if (atom >= G_PASTE_SPECIAL_ATOM_FIRST && atom < G_PASTE_SPECIAL_ATOM_LAST)
+    {
+        g_autofree guchar *data = NULL;
+        guint64 length = 0;
+        const gchar *str = g_paste_item_get_special_value (item, atom);
+
+        if (str)
+        {
+            data = g_base64_decode (str, &length);
+        }
+        else
+        {
+            str = g_paste_item_get_value (item);
+            length = strlen (str);
+            data = copy_str_as_uchars (str, length);
+        }
+
+        gtk_selection_data_set (selection_data, g_paste_special_atom_get (atom), 8, data, length);
+    }
 }
 
 static void
 g_paste_clipboard_get_clipboard_data (GtkClipboard     *clipboard G_GNUC_UNUSED,
                                       GtkSelectionData *selection_data,
-                                      guint             info G_GNUC_UNUSED,
+                                      guint32           info      G_GNUC_UNUSED,
                                       gpointer          user_data_or_owner)
 {
-    g_return_if_fail (G_PASTE_IS_ITEM (user_data_or_owner));
+    g_return_if_fail (_G_PASTE_IS_ITEM (user_data_or_owner));
 
     GPasteItem *item = G_PASTE_ITEM (user_data_or_owner);
 
-    GdkAtom targets[1] = { gtk_selection_data_get_target (selection_data) };
+    GdkAtom target = gtk_selection_data_get_target (selection_data);
+    GdkAtom targets[1] = { target };
+
+    if (_G_PASTE_IS_IMAGE_ITEM (item))
+    {
+        if (gtk_targets_include_image (targets, 1, TRUE))
+            gtk_selection_data_set_pixbuf (selection_data, g_paste_image_item_get_image (G_PASTE_IMAGE_ITEM (item)));
+        return;
+    }
+    else if (_G_PASTE_IS_URIS_ITEM (item))
+    {
+        if (gtk_targets_include_uri (targets, 1))
+        {
+            const gchar * const *uris = g_paste_uris_item_get_uris (G_PASTE_URIS_ITEM (item));
+
+            gtk_selection_data_set_uris (selection_data, (GStrv) uris);
+            return;
+        }
+    }
+
+    for (GPasteSpecialAtom a = G_PASTE_SPECIAL_ATOM_FIRST; a < G_PASTE_SPECIAL_ATOM_LAST; ++a)
+    {
+        if (target == g_paste_special_atom_get (a))
+        {
+            _get_clipboard_data_from_special_atom (selection_data, item, a);
+            return;
+        }
+    }
 
     /* The content is requested as text */
     if (gtk_targets_include_text (targets, 1))
         gtk_selection_data_set_text (selection_data, g_paste_item_get_real_value (item), -1);
-    else if (G_PASTE_IS_IMAGE_ITEM (item))
-    {
-        if (gtk_targets_include_image (targets, 1, TRUE))
-            gtk_selection_data_set_pixbuf (selection_data, g_paste_image_item_get_image (G_PASTE_IMAGE_ITEM (item)));
-    }
-    /* The content is requested as uris */
-    else
-    {
-        g_return_if_fail (G_PASTE_IS_URIS_ITEM (item));
-
-        const gchar * const *uris = g_paste_uris_item_get_uris (G_PASTE_URIS_ITEM (item));
-
-        if (gtk_targets_include_uri (targets, 1))
-            gtk_selection_data_set_uris (selection_data, (GStrv) uris);
-        /* The content is requested as special gnome-copied-files by nautilus */
-        else
-        {
-            G_PASTE_CLEANUP_STRING_FREE GString *copy_string = g_string_new ("copy");
-            guint length = g_strv_length ((GStrv) uris);
-
-            for (guint i = 0; i < length; ++i)
-                g_string_append_printf (copy_string, "\n%s", uris[i]);
-
-            gchar *str = copy_string->str;
-            length = copy_string->len + 1;
-            g_autofree guchar *copy_files_data = g_new (guchar, length);
-            for (guint i = 0; i < length; ++i)
-                copy_files_data[i] = (guchar) str[i];
-            gtk_selection_data_set (selection_data, g_paste_clipboard_copy_files_target, 8, copy_files_data, length);
-        }
-    }
 }
 
 static void
@@ -345,30 +416,47 @@ g_paste_clipboard_clear_clipboard_data (GtkClipboard *clipboard G_GNUC_UNUSED,
     g_object_unref (user_data_or_owner);
 }
 
-static void
-g_paste_clipboard_private_select_uris (GPasteClipboardPrivate *priv,
-                                       GPasteUrisItem         *item)
+/**
+ * g_paste_clipboard_clear:
+ * @self: a #GPasteClipboard instance
+ *
+ * Clears the content of the clipboard
+ */
+G_PASTE_VISIBLE void
+g_paste_clipboard_clear (GPasteClipboard *self)
 {
-    GtkClipboard *real = priv->real;
-    g_autoptr (GtkTargetList) target_list = gtk_target_list_new (NULL, 0);
+    g_return_if_fail (_G_PASTE_IS_CLIPBOARD (self));
 
-    g_paste_clipboard_private_set_text (priv, g_paste_item_get_real_value (G_PASTE_ITEM (item)));
+    GPasteClipboardPrivate *priv = g_paste_clipboard_get_instance_private (self);
 
-    gtk_target_list_add_text_targets (target_list, 0);
-    gtk_target_list_add_uri_targets (target_list, 0);
-    gtk_target_list_add (target_list, g_paste_clipboard_copy_files_target, 0, 0);
+    /* We're already clear, don't enter an infinite event loop */
+    if (!priv->text && !priv->image_checksum)
+        return;
 
-    gint n_targets;
-    GtkTargetEntry *targets = gtk_target_table_new_from_list (target_list, &n_targets);
-    gtk_clipboard_set_with_owner (real,
-                                  targets,
-                                  n_targets,
-                                  g_paste_clipboard_get_clipboard_data,
-                                  g_paste_clipboard_clear_clipboard_data,
-                                  g_object_ref (item));
-    gtk_clipboard_store (real);
+    g_debug("%s: clear", _g_paste_clipboard_private_target_name (priv));
 
-    gtk_target_table_free (targets, n_targets);
+    g_clear_pointer (&priv->text, g_free);
+    g_clear_pointer (&priv->image_checksum, g_free);
+
+    gtk_clipboard_clear (priv->real);
+}
+
+/**
+ * g_paste_clipboard_store:
+ * @self: a #GPasteClipboard instance
+ *
+ * Store the contents of the clipboard before exiting
+ */
+G_PASTE_VISIBLE void
+g_paste_clipboard_store (GPasteClipboard *self)
+{
+    g_return_if_fail (_G_PASTE_IS_CLIPBOARD (self));
+
+    GPasteClipboardPrivate *priv = g_paste_clipboard_get_instance_private (self);
+
+    g_debug("%s: store", _g_paste_clipboard_private_target_name (priv));
+
+    gtk_clipboard_store (priv->real);
 }
 
 /**
@@ -382,9 +470,9 @@ g_paste_clipboard_private_select_uris (GPasteClipboardPrivate *priv,
 G_PASTE_VISIBLE const gchar *
 g_paste_clipboard_get_image_checksum (const GPasteClipboard *self)
 {
-    g_return_val_if_fail (G_PASTE_IS_CLIPBOARD (self), NULL);
+    g_return_val_if_fail (_G_PASTE_IS_CLIPBOARD (self), NULL);
 
-    GPasteClipboardPrivate *priv = g_paste_clipboard_get_instance_private (self);
+    const GPasteClipboardPrivate *priv = _g_paste_clipboard_get_instance_private (self);
 
     return priv->image_checksum;
 }
@@ -409,6 +497,8 @@ g_paste_clipboard_private_select_image (GPasteClipboardPrivate *priv,
 
     GtkClipboard *real = priv->real;
 
+    g_debug("%s: select image", _g_paste_clipboard_private_target_name (priv));
+
     g_paste_clipboard_private_set_image_checksum (priv, checksum);
     gtk_clipboard_set_image (real, image);
 }
@@ -435,21 +525,12 @@ g_paste_clipboard_on_image_ready (GtkClipboard *clipboard G_GNUC_UNUSED,
     }
 
     GPasteClipboardPrivate *priv = g_paste_clipboard_get_instance_private (self);
+    g_autofree gchar *checksum = g_paste_util_compute_checksum (image);
 
-    g_autofree gchar *checksum = g_compute_checksum_for_data (G_CHECKSUM_SHA256,
-                                                                        (guchar *) gdk_pixbuf_get_pixels (image),
-                                                                        -1);
-
-    if (g_strcmp0 (checksum, priv->image_checksum))
-    {
-        g_paste_clipboard_private_select_image (priv,
-                                                image,
-                                                checksum);
-    }
+    if (g_paste_str_equal (checksum, priv->image_checksum))
+        g_clear_object (&image);
     else
-    {
-        image = NULL;
-    }
+        g_paste_clipboard_private_select_image (priv, image, checksum);
 
     if (data->callback)
         data->callback (self, image, data->user_data);
@@ -462,17 +543,15 @@ g_paste_clipboard_on_image_ready (GtkClipboard *clipboard G_GNUC_UNUSED,
  * @user_data: user data to pass to @callback
  *
  * Put the image from the intern GtkClipboard in the #GPasteClipboard
- *
- * Returns:
  */
 G_PASTE_VISIBLE void
 g_paste_clipboard_set_image (GPasteClipboard             *self,
                              GPasteClipboardImageCallback callback,
                              gpointer                     user_data)
 {
-    g_return_if_fail (G_PASTE_IS_CLIPBOARD (self));
+    g_return_if_fail (_G_PASTE_IS_CLIPBOARD (self));
 
-    GPasteClipboardPrivate *priv = g_paste_clipboard_get_instance_private (self);
+    const GPasteClipboardPrivate *priv = _g_paste_clipboard_get_instance_private (self);
     GPasteClipboardImageCallbackData *data = g_new (GPasteClipboardImageCallbackData, 1);
 
     data->self = self;
@@ -491,42 +570,75 @@ g_paste_clipboard_set_image (GPasteClipboard             *self,
  *
  * Put the value of the item into the #GPasteClipbaord and the intern GtkClipboard
  *
- * Returns:
+ * Returns: %FALSE if the item was invalid, %TRUE otherwise
  */
-G_PASTE_VISIBLE void
-g_paste_clipboard_select_item (GPasteClipboard  *self,
-                               const GPasteItem *item)
+G_PASTE_VISIBLE gboolean
+g_paste_clipboard_select_item (GPasteClipboard *self,
+                               GPasteItem      *item)
 {
-    g_return_if_fail (G_PASTE_IS_CLIPBOARD (self));
-    g_return_if_fail (G_PASTE_IS_ITEM (item));
+    g_return_val_if_fail (_G_PASTE_IS_CLIPBOARD (self), FALSE);
+    g_return_val_if_fail (_G_PASTE_IS_ITEM (item), FALSE);
 
     GPasteClipboardPrivate *priv = g_paste_clipboard_get_instance_private (self);
+    GtkClipboard *real = priv->real;
+    g_autoptr (GtkTargetList) target_list = gtk_target_list_new (NULL, 0);
 
-    if (G_PASTE_IS_IMAGE_ITEM (item))
+    g_debug("%s: select item", _g_paste_clipboard_private_target_name (priv));
+
+    if (_G_PASTE_IS_IMAGE_ITEM (item))
     {
-        GPasteImageItem *image_item = G_PASTE_IMAGE_ITEM (item);
-        const gchar *checksum = g_paste_image_item_get_checksum (image_item);
-
-        if (g_strcmp0 (checksum, priv->image_checksum))
-        {
-            g_paste_clipboard_private_select_image (priv,
-                                                    g_paste_image_item_get_image (image_item),
-                                                    checksum);
-        }
+        gtk_target_list_add_image_targets (target_list, 0, FALSE);
+        g_paste_clipboard_private_set_image_checksum (priv, g_paste_image_item_get_checksum (G_PASTE_IMAGE_ITEM (item)));
     }
     else
     {
-        const gchar *text = g_paste_item_get_real_value (item);
+        gtk_target_list_add_text_targets (target_list, 0);
+        if (_G_PASTE_IS_URIS_ITEM (item))
+            gtk_target_list_add_uri_targets (target_list, 0);
+        g_paste_clipboard_private_set_text (priv, g_paste_item_get_real_value (item));
+    }
 
-        if (g_strcmp0 (text, priv->text))
-        {
-            if (G_PASTE_IS_URIS_ITEM (item))
-                g_paste_clipboard_private_select_uris (priv, G_PASTE_URIS_ITEM (item));
-            else  if (G_PASTE_IS_TEXT_ITEM (item))
-                g_paste_clipboard_select_text (self, text);
-            else
-                g_assert_not_reached ();
-        }
+    for (const GSList *sv = g_paste_item_get_special_values (item); sv; sv = sv->next)
+    {
+        const GPasteSpecialValue *v = sv->data;
+        gtk_target_list_add (target_list, g_paste_special_atom_get (v->mime), 0, 0);
+    }
+
+    gint32 n_targets;
+    GtkTargetEntry *targets = gtk_target_table_new_from_list (target_list, &n_targets);
+    gtk_clipboard_set_with_owner (real,
+                                  targets,
+                                  n_targets,
+                                  g_paste_clipboard_get_clipboard_data,
+                                  g_paste_clipboard_clear_clipboard_data,
+                                  g_object_ref ((GObject *) item));
+    gtk_target_table_free (targets, n_targets);
+
+    return TRUE;
+}
+
+/**
+ * g_paste_clipboard_ensure_not_empty:
+ * @self: a #GPasteClipboard instance
+ * @history: a #GPasteHistory instance
+ *
+ * Ensure the clipboard has some contents (as long as the history's not empty)
+ */
+G_PASTE_VISIBLE void
+g_paste_clipboard_ensure_not_empty (GPasteClipboard *self,
+                                    GPasteHistory   *history)
+{
+    g_return_if_fail (_G_PASTE_IS_CLIPBOARD (self));
+    g_return_if_fail (_G_PASTE_IS_HISTORY (history));
+
+    const GList *hist = g_paste_history_get_history (history);
+
+    if (hist)
+    {
+        GPasteItem *item = hist->data;
+
+        if (!g_paste_clipboard_select_item (self, item))
+            g_paste_history_remove (history, 0);
     }
 }
 
@@ -536,6 +648,9 @@ g_paste_clipboard_owner_change (GtkClipboard        *clipboard G_GNUC_UNUSED,
                                 gpointer             user_data)
 {
     GPasteClipboard *self = user_data;
+    GPasteClipboardPrivate *priv = g_paste_clipboard_get_instance_private (self);
+
+    g_debug("%s: owner change", _g_paste_clipboard_private_target_name (priv));
 
     g_signal_emit (self,
 		   signals[OWNER_CHANGE],
@@ -550,9 +665,9 @@ g_paste_clipboard_fake_event_finish_text (GtkClipboard *clipboard G_GNUC_UNUSED,
                                           gpointer      user_data)
 {
     GPasteClipboard *self = user_data;
-    GPasteClipboardPrivate *priv = g_paste_clipboard_get_instance_private (self);
+    const GPasteClipboardPrivate *priv = _g_paste_clipboard_get_instance_private (self);
 
-    if (g_strcmp0 (text, priv->text))
+    if (!g_paste_str_equal (text, priv->text))
         g_paste_clipboard_owner_change (NULL, NULL, self);
 }
 
@@ -562,10 +677,10 @@ g_paste_clipboard_fake_event_finish_image (GtkClipboard *clipboard G_GNUC_UNUSED
                                            gpointer      user_data)
 {
     GPasteClipboard *self = user_data;
-    GPasteClipboardPrivate *priv = g_paste_clipboard_get_instance_private (self);
+    const GPasteClipboardPrivate *priv = _g_paste_clipboard_get_instance_private (self);
     g_autofree gchar *checksum = g_paste_util_compute_checksum (image);
 
-    if (g_strcmp0 (checksum, priv->image_checksum))
+    if (!g_paste_str_equal (checksum, priv->image_checksum))
         g_paste_clipboard_owner_change (NULL, NULL, self);
 
     g_object_unref (image);
@@ -575,7 +690,7 @@ static gboolean
 g_paste_clipboard_fake_event (gpointer user_data)
 {
     GPasteClipboard *self = user_data;
-    GPasteClipboardPrivate *priv = g_paste_clipboard_get_instance_private (self);
+    const GPasteClipboardPrivate *priv = _g_paste_clipboard_get_instance_private (self);
 
     if (priv->text)
         gtk_clipboard_request_text (priv->real, g_paste_clipboard_fake_event_finish_text, self);
@@ -590,11 +705,11 @@ g_paste_clipboard_fake_event (gpointer user_data)
 static void
 g_paste_clipboard_dispose (GObject *object)
 {
-    GPasteClipboardPrivate *priv = g_paste_clipboard_get_instance_private (G_PASTE_CLIPBOARD (object));
+    const GPasteClipboardPrivate *priv = _g_paste_clipboard_get_instance_private (G_PASTE_CLIPBOARD (object));
 
     if (priv->settings)
     {
-        g_signal_handler_disconnect (priv->real, priv->owner_change_signal);
+        g_signal_handler_disconnect (priv->real, priv->c_signals[C_OWNER_CHANGE]);
         g_clear_object (&priv->settings);
     }
 
@@ -604,7 +719,7 @@ g_paste_clipboard_dispose (GObject *object)
 static void
 g_paste_clipboard_finalize (GObject *object)
 {
-    GPasteClipboardPrivate *priv = g_paste_clipboard_get_instance_private (G_PASTE_CLIPBOARD (object));
+    const GPasteClipboardPrivate *priv = _g_paste_clipboard_get_instance_private (G_PASTE_CLIPBOARD (object));
 
     g_free (priv->text);
     g_free (priv->image_checksum);
@@ -620,6 +735,15 @@ g_paste_clipboard_class_init (GPasteClipboardClass *klass)
     object_class->dispose = g_paste_clipboard_dispose;
     object_class->finalize = g_paste_clipboard_finalize;
 
+    /**
+     * GPasteClipboard::owner-change:
+     * @clipboard: the object on which the signal was emitted
+     * @event: (type Gdk.EventOwnerChange): the @GdkEventOwnerChange event
+     *
+     * The "owner-change" signal is emitted when GPaste receives an
+     * event that indicates that the ownership of the selection
+     * associated with @clipboard has changed.
+     */
     signals[OWNER_CHANGE] = g_signal_new ("owner-change",
                                           G_PASTE_TYPE_CLIPBOARD,
                                           G_SIGNAL_RUN_FIRST,
@@ -633,42 +757,23 @@ g_paste_clipboard_class_init (GPasteClipboardClass *klass)
 }
 
 static void
-g_paste_clipboard_init (GPasteClipboard *self)
+g_paste_clipboard_init (GPasteClipboard *self G_GNUC_UNUSED)
 {
-    GPasteClipboardPrivate *priv = g_paste_clipboard_get_instance_private (self);
-
-    priv->text = NULL;
-    priv->image_checksum = NULL;
 }
 
-/**
- * g_paste_clipboard_new:
- * @target: the GdkAtom representating the GtkClipboard we're abstracting
- * @settings: a #GPasteSettings instance
- *
- * Create a new instance of #GPasteClipboard
- *
- * Returns: a newly allocated #GPasteClipboard
- *          free it with g_object_unref
- */
-G_PASTE_VISIBLE GPasteClipboard *
-g_paste_clipboard_new (GdkAtom         target,
-                       GPasteSettings *settings)
+static GPasteClipboard *
+_g_paste_clipboard_new (GPasteSettings *settings,
+                        GdkAtom         target)
 {
-    g_return_val_if_fail (G_PASTE_IS_SETTINGS (settings), NULL);
-
     GPasteClipboard *self = g_object_new (G_PASTE_TYPE_CLIPBOARD, NULL);
     GPasteClipboardPrivate *priv = g_paste_clipboard_get_instance_private (self);
-
-    priv->target = target;
-    priv->settings = g_object_ref (settings);
-
     GtkClipboard *real = priv->real = gtk_clipboard_get (target);
 
-    priv->owner_change_signal = g_signal_connect (real,
-                                                  "owner-change",
-                                                  G_CALLBACK (g_paste_clipboard_owner_change),
-                                                  self);
+    priv->settings = g_object_ref (settings);
+    priv->c_signals[C_OWNER_CHANGE] = g_signal_connect (real,
+                                                        "owner-change",
+                                                        G_CALLBACK (g_paste_clipboard_owner_change),
+                                                        self);
 
     if (!gdk_display_request_selection_notification (gdk_display_get_default (), target))
     {
@@ -678,3 +783,37 @@ g_paste_clipboard_new (GdkAtom         target,
 
     return self;
 }
+
+/**
+ * g_paste_clipboard_new_clipboard:
+ * @settings: a #GPasteSettings instance
+ *
+ * Create a new instance of #GPasteClipboard
+ *
+ * Returns: a newly allocated #GPasteClipboard
+ *          free it with g_object_unref
+ */
+G_PASTE_VISIBLE GPasteClipboard *
+g_paste_clipboard_new_clipboard (GPasteSettings *settings)
+{
+    g_return_val_if_fail (_G_PASTE_IS_SETTINGS (settings), NULL);
+
+    return _g_paste_clipboard_new (settings, GDK_SELECTION_CLIPBOARD);
+}
+
+/**
+ * g_paste_clipboard_new_primary:
+ * @settings: a #GPasteSettings instance
+ *
+ * Create a new instance of #GPasteClipboard
+ *
+ * Returns: a newly allocated #GPasteClipboard
+ *          free it with g_object_unref
+ */
+G_PASTE_VISIBLE GPasteClipboard *
+g_paste_clipboard_new_primary (GPasteSettings *settings)
+{
+    g_return_val_if_fail (_G_PASTE_IS_SETTINGS (settings), NULL);
+
+    return _g_paste_clipboard_new (settings, GDK_SELECTION_PRIMARY);
+}
