@@ -1,20 +1,7 @@
 /*
- *      This file is part of GPaste.
+ * This file is part of GPaste.
  *
- *      Copyright 2013-2015 Marc-Antoine Perennou <Marc-Antoine@Perennou.com>
- *
- *      GPaste is free software: you can redistribute it and/or modify
- *      it under the terms of the GNU General Public License as published by
- *      the Free Software Foundation, either version 3 of the License, or
- *      (at your option) any later version.
- *
- *      GPaste is distributed in the hope that it will be useful,
- *      but WITHOUT ANY WARRANTY; without even the implied warranty of
- *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *      GNU General Public License for more details.
- *
- *      You should have received a copy of the GNU General Public License
- *      along with GPaste.  If not, see <http://www.gnu.org/licenses/>.
+ * Copyright (c) 2010-2018, Marc-Antoine Perennou <Marc-Antoine@Perennou.com>
  */
 
 #if !defined (__G_PASTE_H_INSIDE__) && !defined (G_PASTE_COMPILATION)
@@ -24,36 +11,16 @@
 #ifndef __G_PASTE_GDBUS_MACROS_H__
 #define __G_PASTE_GDBUS_MACROS_H__
 
-#include <glib.h>
-#include <gio/gio.h>
+#include <gpaste-gdbus-defines.h>
+#include <gpaste-util.h>
 
 G_BEGIN_DECLS
 
-/*********************/
-/* Custom Extractors */
-/*********************/
-
-#ifdef __G_PASTE_NEEDS_AU__
-static guint32 *
-g_paste_dbus_get_au_result (GVariant *variant,
-                            gsize    *len)
-{
-    gsize _len;
-    const guint32 *r = g_variant_get_fixed_array (variant, &_len, sizeof (guint32));
-    guint32 *ret = g_memdup (r, _len * sizeof (guint32));
-
-    if (len)
-        *len = _len;
-
-    return ret;
-}
-#endif
-
 /***************/
 /* Constructor */
 /***************/
 
-#define CUSTOM_PROXY_NEW_ASYNC(TYPE, BUS_ID)                                           \
+#define CUSTOM_PROXY_NEW_ASYNC(TYPE, BUS_ID, BUS_NAME)                                 \
     g_async_initable_new_async (G_PASTE_TYPE_##TYPE,                                   \
                                 G_PRIORITY_DEFAULT,                                    \
                                 NULL, /* cancellable */                                \
@@ -61,7 +28,7 @@ g_paste_dbus_get_au_result (GVariant *variant,
                                 user_data,                                             \
                                 "g-bus-type",       G_BUS_TYPE_SESSION,                \
                                 "g-flags",          G_DBUS_PROXY_FLAGS_NONE,           \
-                                "g-name",           G_PASTE_##BUS_ID##_BUS_NAME,       \
+                                "g-name",           BUS_NAME,                          \
                                 "g-object-path",    G_PASTE_##BUS_ID##_OBJECT_PATH,    \
                                 "g-interface-name", G_PASTE_##BUS_ID##_INTERFACE_NAME, \
                                 NULL)
@@ -76,13 +43,13 @@ g_paste_dbus_get_au_result (GVariant *variant,
                                                  error);                    \
     return (self) ? G_PASTE_##TYPE (self) : NULL;
 
-#define CUSTOM_PROXY_NEW(TYPE, BUS_ID)                                                       \
+#define CUSTOM_PROXY_NEW(TYPE, BUS_ID, BUS_NAME)                                             \
     GInitable *self = g_initable_new (G_PASTE_TYPE_##TYPE,                                   \
                                       NULL, /* cancellable */                                \
                                       error,                                                 \
                                       "g-bus-type",       G_BUS_TYPE_SESSION,                \
                                       "g-flags",          G_DBUS_PROXY_FLAGS_NONE,           \
-                                      "g-name",           G_PASTE_##BUS_ID##_BUS_NAME,       \
+                                      "g-name",           BUS_NAME,                          \
                                       "g-object-path",    G_PASTE_##BUS_ID##_OBJECT_PATH,    \
                                       "g-interface-name", G_PASTE_##BUS_ID##_INTERFACE_NAME, \
                                       NULL);                                                 \
@@ -107,7 +74,7 @@ g_paste_dbus_get_au_result (GVariant *variant,
 /*****************************/
 
 #define DBUS_CALL_ASYNC_FULL(TYPE_CHECKER, decl, method, params, n_params) \
-    g_return_if_fail (G_PASTE_IS_##TYPE_CHECKER (self));                   \
+    g_return_if_fail (_G_PASTE_IS_##TYPE_CHECKER (self));                   \
     decl;                                                                  \
     g_dbus_proxy_call (G_DBUS_PROXY(self),                                 \
                        method,                                             \
@@ -149,7 +116,7 @@ g_paste_dbus_get_au_result (GVariant *variant,
     DBUS_RETURN (if_fail, extract_and_return_answer)
 
 #define DBUS_ASYNC_FINISH_WITH_RETURN(TYPE_CHECKER, if_fail, extract_and_return_answer)                \
-    DBUS_ASYNC_FINISH_FULL (g_return_val_if_fail (G_PASTE_IS_##TYPE_CHECKER (self), if_fail);          \
+    DBUS_ASYNC_FINISH_FULL (g_return_val_if_fail (_G_PASTE_IS_##TYPE_CHECKER (self), if_fail);          \
                             g_return_val_if_fail (G_IS_ASYNC_RESULT (result), if_fail);                \
                             g_return_val_if_fail (!error || !(*error), if_fail),                       \
                             if_fail,                                                                   \
@@ -161,13 +128,16 @@ g_paste_dbus_get_au_result (GVariant *variant,
 /***********************************/
 
 #define DBUS_ASYNC_FINISH_NO_RETURN_BASE(TYPE_CHECKER)                           \
-    DBUS_ASYNC_FINISH_FULL (g_return_if_fail (G_PASTE_IS_##TYPE_CHECKER (self)); \
+    DBUS_ASYNC_FINISH_FULL (g_return_if_fail (_G_PASTE_IS_##TYPE_CHECKER (self)); \
                             g_return_if_fail (G_IS_ASYNC_RESULT (result));       \
                             g_return_if_fail (!error || !(*error)), ;, {})
 
 #define DBUS_ASYNC_FINISH_RET_BOOL_BASE(TYPE_CHECKER) \
     DBUS_ASYNC_FINISH_WITH_RETURN (TYPE_CHECKER, FALSE, return g_variant_get_boolean (variant))
 
+#define DBUS_ASYNC_FINISH_RET_UINT64_BASE(TYPE_CHECKER) \
+    DBUS_ASYNC_FINISH_WITH_RETURN (TYPE_CHECKER, 0, return g_variant_get_uint64 (variant))
+
 #define DBUS_ASYNC_FINISH_RET_UINT32_BASE(TYPE_CHECKER) \
     DBUS_ASYNC_FINISH_WITH_RETURN (TYPE_CHECKER, 0, return g_variant_get_uint32 (variant))
 
@@ -177,8 +147,11 @@ g_paste_dbus_get_au_result (GVariant *variant,
 #define DBUS_ASYNC_FINISH_RET_STRV_BASE(TYPE_CHECKER) \
     DBUS_ASYNC_FINISH_WITH_RETURN (TYPE_CHECKER, NULL, return g_variant_dup_strv (variant, NULL))
 
+#define DBUS_ASYNC_FINISH_RET_AT_BASE(TYPE_CHECKER, len) \
+    DBUS_ASYNC_FINISH_WITH_RETURN (TYPE_CHECKER, NULL, return g_paste_util_get_dbus_at_result (variant, len))
+
 #define DBUS_ASYNC_FINISH_RET_AU_BASE(TYPE_CHECKER, len) \
-    DBUS_ASYNC_FINISH_WITH_RETURN (TYPE_CHECKER, NULL, return g_paste_dbus_get_au_result (variant, len))
+    DBUS_ASYNC_FINISH_WITH_RETURN (TYPE_CHECKER, NULL, return g_paste_util_get_dbus_au_result (variant, len))
 
 /****************************/
 /* Methods / Sync / General */
@@ -201,14 +174,14 @@ g_paste_dbus_get_au_result (GVariant *variant,
 /****************************************/
 
 #define DBUS_CALL_NO_RETURN_BASE(TYPE_CHECKER, decl, method, params, n_params) \
-    DBUS_CALL_FULL (g_return_if_fail (G_PASTE_IS_##TYPE_CHECKER (self)), decl, method, params, n_params, ;, {})
+    DBUS_CALL_FULL (g_return_if_fail (_G_PASTE_IS_##TYPE_CHECKER (self)), decl, method, params, n_params, ;, {})
 
 /******************************************/
 /* Methods / Sync / General - With return */
 /******************************************/
 
 #define DBUS_CALL_WITH_RETURN_FULL_BASE(TYPE_CHECKER, decl, method, params, n_params, if_fail, variant_extract, pre_extract)                        \
-    DBUS_CALL_FULL (g_return_val_if_fail (G_PASTE_IS_##TYPE_CHECKER (self), if_fail), decl, method, params, n_params, if_fail, \
+    DBUS_CALL_FULL (g_return_val_if_fail (_G_PASTE_IS_##TYPE_CHECKER (self), if_fail), decl, method, params, n_params, if_fail, \
                     pre_extract;                                                                                                                    \
                     variant_extract)
 
@@ -244,14 +217,20 @@ g_paste_dbus_get_au_result (GVariant *variant,
 #define DBUS_CALL_NO_PARAM_BASE(TYPE_CHECKER, method, if_fail, variant_extract) \
     DBUS_CALL_WITH_RETURN_BASE (TYPE_CHECKER, {}, method, NULL, 0, if_fail, variant_extract)
 
+#define DBUS_CALL_NO_PARAM_RET_STRING_BASE(TYPE_CHECKER, method) \
+    DBUS_CALL_NO_PARAM_BASE(TYPE_CHECKER, method, NULL, return g_variant_dup_string (variant, NULL)) \
+
 #define DBUS_CALL_NO_PARAM_RET_STRV_BASE(TYPE_CHECKER, method) \
     DBUS_CALL_NO_PARAM_BASE(TYPE_CHECKER, method, NULL, return g_variant_dup_strv (variant, NULL)) \
 
-#define DBUS_CALL_NO_PARAM_RET_UINT32_BASE(TYPE_CHECKER, method) \
-    DBUS_CALL_NO_PARAM_BASE(TYPE_CHECKER, method, 0, return g_variant_get_uint32 (variant)) \
+#define DBUS_CALL_ONE_PARAMV_RET_AT_BASE(TYPE_CHECKER, method, paramv, len) \
+    DBUS_CALL_WITH_RETURN_BASE(TYPE_CHECKER, {}, method, &paramv, 1, NULL, return g_paste_util_get_dbus_at_result (variant, len))
 
 #define DBUS_CALL_ONE_PARAMV_RET_AU_BASE(TYPE_CHECKER, method, paramv, len) \
-    DBUS_CALL_WITH_RETURN_BASE(TYPE_CHECKER, {}, method, &paramv, 1, NULL, return g_paste_dbus_get_au_result (variant, len))
+    DBUS_CALL_WITH_RETURN_BASE(TYPE_CHECKER, {}, method, &paramv, 1, NULL, return g_paste_util_get_dbus_au_result (variant, len))
+
+#define DBUS_CALL_ONE_PARAMV_RET_STRV_BASE(TYPE_CHECKER, method, paramv) \
+    DBUS_CALL_WITH_RETURN_BASE(TYPE_CHECKER, {}, method, &paramv, 1, NULL, return g_variant_dup_strv (variant, NULL))
 
 /******************************************************/
 /* Methods / Sync / General - With return - One param */
@@ -270,11 +249,14 @@ g_paste_dbus_get_au_result (GVariant *variant,
 #define DBUS_CALL_ONE_PARAM_RET_BOOL_BASE(TYPE_CHECKER, param_type, param_name, method) \
     DBUS_CALL_ONE_PARAM_BASE(TYPE_CHECKER, param_type, param_name, method, FALSE, return g_variant_get_boolean (variant))
 
+#define DBUS_CALL_ONE_PARAM_RET_UINT64_BASE(TYPE_CHECKER, param_type, param_name, method) \
+    DBUS_CALL_ONE_PARAM_BASE(TYPE_CHECKER, param_type, param_name, method, 0, return g_variant_get_uint64 (variant))
+
 #define DBUS_CALL_ONE_PARAM_RET_STRING_BASE(TYPE_CHECKER, param_type, param_name, method) \
     DBUS_CALL_ONE_PARAM_BASE(TYPE_CHECKER, param_type, param_name, method, NULL, return g_variant_dup_string (variant, NULL /* length */))
 
-#define DBUS_CALL_ONE_PARAM_RET_AU_BASE(TYPE_CHECKER, param_type, param_name, method, len) \
-    DBUS_CALL_ONE_PARAM_BASE(TYPE_CHECKER, param_type, param_name, method, NULL, return g_paste_dbus_get_au_result (variant, len))
+#define DBUS_CALL_ONE_PARAM_RET_AT_BASE(TYPE_CHECKER, param_type, param_name, method, len) \
+    DBUS_CALL_ONE_PARAM_BASE(TYPE_CHECKER, param_type, param_name, method, NULL, return g_paste_util_get_dbus_at_result (variant, len))
 
 /****************************************************/
 /* Methods / Sync / Impl - With return - Two params */
@@ -283,6 +265,9 @@ g_paste_dbus_get_au_result (GVariant *variant,
 #define DBUS_CALL_TWO_PARAMS_BASE(TYPE_CHECKER, params, method, if_fail, variant_extract) \
     DBUS_CALL_WITH_RETURN_BASE (TYPE_CHECKER, {}, method, params, 2, if_fail, variant_extract)
 
+#define DBUS_CALL_TWO_PARAMS_RET_UINT64_BASE(TYPE_CHECKER, params, method) \
+    DBUS_CALL_TWO_PARAMS_BASE(TYPE_CHECKER, params, method, 0, return g_variant_get_uint64 (variant))
+
 #define DBUS_CALL_TWO_PARAMS_RET_UINT32_BASE(TYPE_CHECKER, params, method) \
     DBUS_CALL_TWO_PARAMS_BASE(TYPE_CHECKER, params, method, 0, return g_variant_get_uint32 (variant))
 
@@ -291,7 +276,7 @@ g_paste_dbus_get_au_result (GVariant *variant,
 /************************/
 
 #define DBUS_GET_PROPERTY_INIT(TYPE_CHECKER, property,  _default)                        \
-    g_return_val_if_fail (G_PASTE_IS_##TYPE_CHECKER (self), _default);                   \
+    g_return_val_if_fail (_G_PASTE_IS_##TYPE_CHECKER (self), _default);                   \
     g_autoptr (GVariant) result = g_dbus_proxy_get_cached_property (G_DBUS_PROXY (self), \
                                                                     property);           \
     if (!result)                                                                         \
@@ -310,7 +295,7 @@ g_paste_dbus_get_au_result (GVariant *variant,
 /************************/
 
 #define DBUS_SET_GENERIC_PROPERTY_BASE(TYPE_CHECKER, iface, property, value, vtype)              \
-    g_return_val_if_fail (G_PASTE_IS_##TYPE_CHECKER (self), FALSE);                              \
+    g_return_val_if_fail (_G_PASTE_IS_##TYPE_CHECKER (self), FALSE);                              \
     GVariant *prop[] = {                                                                         \
         g_variant_new_string (iface),                                                            \
         g_variant_new_string (property),                                                         \
