@@ -1,20 +1,7 @@
 /*
- *      This file is part of GPaste.
+ * This file is part of GPaste.
  *
- *      Copyright 2011-2015 Marc-Antoine Perennou <Marc-Antoine@Perennou.com>
- *
- *      GPaste is free software: you can redistribute it and/or modify
- *      it under the terms of the GNU General Public License as published by
- *      the Free Software Foundation, either version 3 of the License, or
- *      (at your option) any later version.
- *
- *      GPaste is distributed in the hope that it will be useful,
- *      but WITHOUT ANY WARRANTY; without even the implied warranty of
- *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *      GNU General Public License for more details.
- *
- *      You should have received a copy of the GNU General Public License
- *      along with GPaste.  If not, see <http://www.gnu.org/licenses/>.
+ * Copyright (c) 2010-2018, Marc-Antoine Perennou <Marc-Antoine@Perennou.com>
  */
 
 #include <gpaste-keybinder.h>
@@ -34,6 +21,13 @@ struct _GPasteKeybinder
     GObject parent_instance;
 };
 
+enum
+{
+    C_ACCEL,
+
+    C_LAST_SIGNAL
+};
+
 typedef struct
 {
     GSList                 *keybindings;
@@ -42,13 +36,14 @@ typedef struct
     GPasteGnomeShellClient *shell_client;
     gboolean                use_shell_client;
     gboolean                grabbing;
-    guint                   retries;
+    guint64                 retries;
 
-    gulong                  accel_signal;
-    guint                   shell_watch;
+    guint64                 shell_watch;
+
+    guint64                 c_signals[C_LAST_SIGNAL];
 } GPasteKeybinderPrivate;
 
-G_DEFINE_TYPE_WITH_PRIVATE (GPasteKeybinder, g_paste_keybinder, G_TYPE_OBJECT)
+G_PASTE_DEFINE_TYPE_WITH_PRIVATE (Keybinder, keybinder, G_TYPE_OBJECT)
 
 /***************************/
 /* Internal grabbing stuff */
@@ -65,7 +60,7 @@ g_paste_keybinder_change_grab_wayland (void)
 #if defined(ENABLE_X_KEYBINDER) && defined (GDK_WINDOWING_X11)
 static void
 g_paste_keybinder_change_grab_x11 (GPasteKeybinding *binding,
-                                   Display          *display,
+                                   GdkDisplay       *display,
                                    gboolean          grab)
 {
     if (!g_paste_keybinding_is_active (binding))
@@ -73,12 +68,13 @@ g_paste_keybinder_change_grab_x11 (GPasteKeybinding *binding,
 
     guchar mask_bits[XIMaskLen (XI_LASTEVENT)] = { 0 };
     XIEventMask mask = { XIAllMasterDevices, sizeof (mask_bits), mask_bits };
+    Display *xdisplay = GDK_DISPLAY_XDISPLAY (display);
 
     XISetMask (mask.mask, XI_KeyPress);
 
-    gdk_error_trap_push ();
+    gdk_x11_display_error_trap_push (display);
 
-    guint mod_masks [] = {
+    guint64 mod_masks [] = {
         0, /* modifier only */
         GDK_MOD2_MASK, /* NumLock */
         GDK_MOD5_MASK, /* ScrollLock */
@@ -91,15 +87,15 @@ g_paste_keybinder_change_grab_x11 (GPasteKeybinding *binding,
 
     Window window = GDK_ROOT_WINDOW ();
     GdkModifierType modifiers = g_paste_keybinding_get_modifiers (binding);
-    const guint *keycodes = g_paste_keybinding_get_keycodes (binding);
+    const guint32 *keycodes = g_paste_keybinding_get_keycodes (binding);
 
-    for (guint i = 0; i < G_N_ELEMENTS (mod_masks); ++i) {
+    for (guint64 i = 0; i < G_N_ELEMENTS (mod_masks); ++i) {
         XIGrabModifiers mods = { mod_masks[i] | modifiers, 0 };
-        for (const guint *keycode = keycodes; *keycode; ++keycode)
+        for (const guint32 *keycode = keycodes; *keycode; ++keycode)
         {
             if (grab)
             {
-                XIGrabKeycode (display,
+                XIGrabKeycode (xdisplay,
                                XIAllMasterDevices,
                                *keycode,
                                window,
@@ -112,7 +108,7 @@ g_paste_keybinder_change_grab_x11 (GPasteKeybinding *binding,
             }
             else
             {
-                XIUngrabKeycode (display,
+                XIUngrabKeycode (xdisplay,
                                  XIAllMasterDevices,
                                  *keycode,
                                  window,
@@ -122,8 +118,8 @@ g_paste_keybinder_change_grab_x11 (GPasteKeybinding *binding,
         }
     }
 
-    gdk_flush ();
-    gdk_error_trap_pop_ignored ();
+    gdk_display_flush (display);
+    gdk_x11_display_error_trap_pop_ignored (display);
 }
 #endif
 
@@ -140,7 +136,7 @@ g_paste_keybinder_change_grab_internal (GPasteKeybinding *binding,
 #endif
 #if defined(ENABLE_X_KEYBINDER) && defined (GDK_WINDOWING_X11)
     if (GDK_IS_X11_DISPLAY (display))
-        g_paste_keybinder_change_grab_x11 (binding, GDK_DISPLAY_XDISPLAY (display), grab);
+        g_paste_keybinder_change_grab_x11 (binding, display, grab);
     else
 #endif
         g_warning ("Unsupported GDK backend, keybinder won't work.");
@@ -150,6 +146,13 @@ g_paste_keybinder_change_grab_internal (GPasteKeybinding *binding,
 /* Wrapper around GPasteKeybinding */
 /***********************************/
 
+enum
+{
+    C_K_REBIND,
+
+    C_K_LAST_SIGNAL
+};
+
 typedef struct
 {
     GPasteKeybinding       *binding;
@@ -159,7 +162,7 @@ typedef struct
 
     guint32                 action;
 
-    gulong                  rebind_signal;
+    guint64                 c_signals[C_K_LAST_SIGNAL];
 } _Keybinding;
 
 static void
@@ -270,12 +273,12 @@ _keybinding_new (GPasteKeybinding       *binding,
 
     k->action = 0;
 
-    g_autofree gchar *detailed_signal = g_strdup_printf ("rebind::%s",
-                                                                   g_paste_keybinding_get_dconf_key (binding));
-    k->rebind_signal = g_signal_connect_swapped (settings,
-                                                 detailed_signal,
-                                                 G_CALLBACK (_keybinding_rebind),
-                                                 k);
+    g_autofree gchar *detailed_signal = g_strdup_printf ("rebind::%s", g_paste_keybinding_get_dconf_key (binding));
+
+    k->c_signals[C_K_REBIND] = g_signal_connect_swapped (settings,
+                                                         detailed_signal,
+                                                         G_CALLBACK (_keybinding_rebind),
+                                                         k);
     return k;
 }
 
@@ -289,7 +292,7 @@ static void
 _keybinding_free (_Keybinding *k)
 {
     _keybinding_ungrab (k);
-    g_signal_handler_disconnect (k->settings, k->rebind_signal);
+    g_signal_handler_disconnect (k->settings, k->c_signals[C_K_REBIND]);
     g_object_unref (k->binding);
     g_object_unref (k->settings);
     g_clear_object (&k->shell_client);
@@ -304,15 +307,13 @@ _keybinding_free (_Keybinding *k)
  * @binding: (transfer full): a #GPasteKeybinding instance
  *
  * Add a new keybinding
- *
- * Returns:
  */
 G_PASTE_VISIBLE void
 g_paste_keybinder_add_keybinding (GPasteKeybinder  *self,
                                   GPasteKeybinding *binding)
 {
-    g_return_if_fail (G_PASTE_IS_KEYBINDER (self));
-    g_return_if_fail (G_PASTE_IS_KEYBINDING (binding));
+    g_return_if_fail (_G_PASTE_IS_KEYBINDER (self));
+    g_return_if_fail (_G_PASTE_IS_KEYBINDING (binding));
 
     GPasteKeybinderPrivate *priv = g_paste_keybinder_get_instance_private (self);
 
@@ -362,9 +363,7 @@ grab_accelerators_cb (GObject      *source_object,
 {
     GPasteKeybinderPrivate *priv = user_data;
     g_autoptr (GError) error = NULL;
-    g_autofree guint *actions = g_paste_gnome_shell_client_grab_accelerators_finish (G_PASTE_GNOME_SHELL_CLIENT (source_object),
-                                                                                               res,
-                                                                                               &error);
+    g_autofree guint32 *actions = g_paste_gnome_shell_client_grab_accelerators_finish (G_PASTE_GNOME_SHELL_CLIENT (source_object), res, &error);
 
     if (error)
     {
@@ -383,7 +382,7 @@ grab_accelerators_cb (GObject      *source_object,
     {
         priv->retries = 0;
 
-        guint index = 0;
+        guint64 index = 0;
         for (GSList *binding = priv->keybindings; binding; binding = g_slist_next (binding))
         {
             _Keybinding *k = binding->data;
@@ -406,7 +405,7 @@ g_paste_keybinder_private_grab_all_gnome_shell (GPasteKeybinderPrivate *priv)
     GPasteGnomeShellAccelerator accels[MAX_BINDINGS + 1];
     GPasteSettings *settings = priv->settings;
 
-    guint index = 0;
+    guint64 index = 0;
     for (GSList *binding = priv->keybindings; binding && index < MAX_BINDINGS; binding = g_slist_next (binding))
     {
         _Keybinding *k = binding->data;
@@ -432,13 +431,11 @@ g_paste_keybinder_private_grab_all_gnome_shell (GPasteKeybinderPrivate *priv)
  * @self: a #GPasteKeybinder instance
  *
  * Activate all the managed keybindings
- *
- * Returns:
  */
 G_PASTE_VISIBLE void
 g_paste_keybinder_activate_all (GPasteKeybinder *self)
 {
-    g_return_if_fail (G_PASTE_IS_KEYBINDER (self));
+    g_return_if_fail (_G_PASTE_IS_KEYBINDER (self));
 
     GPasteKeybinderPrivate *priv = g_paste_keybinder_get_instance_private (self);
 
@@ -465,15 +462,13 @@ g_paste_keybinder_deactivate_keybinding_func (gpointer data,
  * @self: a #GPasteKeybinder instance
  *
  * Deactivate all the managed keybindings
- *
- * Returns:
  */
 G_PASTE_VISIBLE void
 g_paste_keybinder_deactivate_all (GPasteKeybinder *self)
 {
-    g_return_if_fail (G_PASTE_IS_KEYBINDER (self));
+    g_return_if_fail (_G_PASTE_IS_KEYBINDER (self));
 
-    GPasteKeybinderPrivate *priv = g_paste_keybinder_get_instance_private (self);
+    const GPasteKeybinderPrivate *priv = _g_paste_keybinder_get_instance_private (self);
 
     g_slist_foreach (priv->keybindings,
                      g_paste_keybinder_deactivate_keybinding_func,
@@ -492,13 +487,13 @@ g_paste_keybinder_parse_event_wayland (void)
 static gint
 g_paste_keybinder_get_xinput_opcode (Display *display)
 {
-    static gint xinput_opcode = 0;
+    static gint32 xinput_opcode = 0;
 
     if (!xinput_opcode)
     {
-        gint major = 2, minor = 3;
-        gint xinput_error_base;
-        gint xinput_event_base;
+        gint32 major = 2, minor = 3;
+        gint32 xinput_error_base;
+        gint32 xinput_event_base;
 
         if (XQueryExtension (display,
                              "XInputExtension",
@@ -517,7 +512,7 @@ g_paste_keybinder_get_xinput_opcode (Display *display)
 static void
 g_paste_keybinder_parse_event_x11 (XEvent                  *event,
                                    GdkModifierType         *modifiers,
-                                   guint                   *keycode)
+                                   guint64                 *keycode)
 {
     XGenericEventCookie cookie = event->xcookie;
 
@@ -540,21 +535,20 @@ g_paste_keybinder_filter (GdkXEvent *xevent,
                           gpointer   data)
 {
     GPasteKeybinderPrivate *priv = data;
+    GdkDisplay *display = gdk_display_get_default ();
 
-    for (GList *dev = gdk_device_manager_list_devices (gdk_display_get_device_manager (gdk_display_get_default ()),
-                                                       GDK_DEVICE_TYPE_MASTER); dev; dev = g_list_next (dev))
+    for (GList *_seat = gdk_display_list_seats (display); _seat; _seat = g_list_next (_seat))
     {
-        GdkDevice *device = dev->data;
+        GdkSeat *seat = _seat->data;
 
-        if (gdk_device_get_source (device) == GDK_SOURCE_KEYBOARD)
-            gdk_device_ungrab (device, GDK_CURRENT_TIME);
+        if (gdk_seat_get_keyboard (seat))
+            gdk_seat_ungrab (seat);
     }
 
-    gdk_flush ();
+    gdk_display_flush (display);
 
-    GdkDisplay *display = gdk_display_get_default ();
     GdkModifierType modifiers = 0;
-    guint keycode = 0;
+    guint64 keycode = 0;
 
 #ifdef GDK_WINDOWING_WAYLAND
     if (GDK_IS_WAYLAND_DISPLAY (display))
@@ -639,7 +633,7 @@ g_paste_keybinder_dispose (GObject *object)
 
     if (priv->shell_client)
     {
-        g_signal_handler_disconnect (priv->shell_client, priv->accel_signal);
+        g_signal_handler_disconnect (priv->shell_client, priv->c_signals[C_ACCEL]);
         g_clear_object (&priv->shell_client);
     }
 
@@ -709,8 +703,8 @@ G_PASTE_VISIBLE GPasteKeybinder *
 g_paste_keybinder_new (GPasteSettings         *settings,
                        GPasteGnomeShellClient *shell_client)
 {
-    g_return_val_if_fail (G_PASTE_IS_SETTINGS (settings), NULL);
-    g_return_val_if_fail (!shell_client || G_PASTE_IS_GNOME_SHELL_CLIENT (shell_client), NULL);
+    g_return_val_if_fail (_G_PASTE_IS_SETTINGS (settings), NULL);
+    g_return_val_if_fail (!shell_client || _G_PASTE_IS_GNOME_SHELL_CLIENT (shell_client), NULL);
 
     GPasteKeybinder *self = G_PASTE_KEYBINDER (g_object_new (G_PASTE_TYPE_KEYBINDER, NULL));
     GPasteKeybinderPrivate *priv = g_paste_keybinder_get_instance_private (self);
@@ -723,10 +717,10 @@ g_paste_keybinder_new (GPasteSettings         *settings,
 
     if (shell_client)
     {
-        priv->accel_signal = g_signal_connect (shell_client,
-                                               "accelerator-activated",
-                                               G_CALLBACK (on_accelerator_activated),
-                                               priv);
+        priv->c_signals[C_ACCEL] = g_signal_connect (shell_client,
+                                                     "accelerator-activated",
+                                                     G_CALLBACK (on_accelerator_activated),
+                                                     priv);
         priv->shell_watch = g_bus_watch_name (G_BUS_TYPE_SESSION,
                                               G_PASTE_GNOME_SHELL_BUS_NAME,
                                               G_BUS_NAME_WATCHER_FLAGS_NONE,
