@@ -1,48 +1,45 @@
 /*
- *      This file is part of GPaste.
+ * This file is part of GPaste.
  *
- *      Copyright 2011-2015 Marc-Antoine Perennou <Marc-Antoine@Perennou.com>
- *
- *      GPaste is free software: you can redistribute it and/or modify
- *      it under the terms of the GNU General Public License as published by
- *      the Free Software Foundation, either version 3 of the License, or
- *      (at your option) any later version.
- *
- *      GPaste is distributed in the hope that it will be useful,
- *      but WITHOUT ANY WARRANTY; without even the implied warranty of
- *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *      GNU General Public License for more details.
- *
- *      You should have received a copy of the GNU General Public License
- *      along with GPaste.  If not, see <http://www.gnu.org/licenses/>.
+ * Copyright (c) 2010-2018, Marc-Antoine Perennou <Marc-Antoine@Perennou.com>
  */
 
 #include <gpaste-gsettings-keys.h>
-#include <gpaste-settings.h>
+#include <gpaste-util.h>
 
 struct _GPasteSettings
 {
     GObject parent_instance;
 };
 
+enum
+{
+    C_CHANGED,
+    C_SHELL_CHANGED,
+
+    C_LAST_SIGNAL
+};
+
 typedef struct
 {
     GSettings *settings;
     GSettings *shell_settings;
 
-    guint32    element_size;
+    gboolean   close_on_select;
+    guint64    element_size;
     gboolean   growing_lines;
     gchar     *history_name;
     gboolean   images_support;
     gchar     *launch_ui;
     gchar     *make_password;
-    guint32    max_displayed_history_size;
-    guint32    max_history_size;
-    guint32    max_memory_usage;
-    guint32    max_text_item_size;
-    guint32    min_text_item_size;
+    guint64    max_displayed_history_size;
+    guint64    max_history_size;
+    guint64    max_memory_usage;
+    guint64    max_text_item_size;
+    guint64    min_text_item_size;
     gchar     *pop;
     gboolean   primary_to_history;
+    gboolean   rich_text_support;
     gboolean   save_history;
     gchar     *show_history;
     gchar     *sync_clipboard_to_primary;
@@ -55,11 +52,10 @@ typedef struct
 
     gboolean   extension_enabled;
 
-    gulong     changed_signal;
-    gulong     shell_changed_signal;
+    guint64    c_signals[C_LAST_SIGNAL];
 } GPasteSettingsPrivate;
 
-G_DEFINE_TYPE_WITH_PRIVATE (GPasteSettings, g_paste_settings, G_TYPE_OBJECT)
+G_PASTE_DEFINE_TYPE_WITH_PRIVATE (Settings, settings, G_TYPE_OBJECT)
 
 enum
 {
@@ -70,21 +66,21 @@ enum
     LAST_SIGNAL
 };
 
-static guint signals[LAST_SIGNAL] = { 0 };
+static guint64 signals[LAST_SIGNAL] = { 0 };
 
 #define SETTING(name, key, type, setting_type, fail, guards, clear_func, dup_func)                     \
     G_PASTE_VISIBLE type                                                                               \
     g_paste_settings_get_##name (const GPasteSettings *self)                                           \
     {                                                                                                  \
-        g_return_val_if_fail (G_PASTE_IS_SETTINGS ((gpointer) self), fail);                                       \
-        GPasteSettingsPrivate *priv = g_paste_settings_get_instance_private (self); \
+        g_return_val_if_fail (_G_PASTE_IS_SETTINGS ((gpointer) self), fail);                           \
+        const GPasteSettingsPrivate *priv = _g_paste_settings_get_instance_private (self);             \
         return priv->name;                                                                             \
     }                                                                                                  \
     G_PASTE_VISIBLE void                                                                               \
     g_paste_settings_reset_##name (GPasteSettings *self)                                               \
     {                                                                                                  \
-        g_return_if_fail (G_PASTE_IS_SETTINGS (self));                                                 \
-        GPasteSettingsPrivate *priv = g_paste_settings_get_instance_private (self); \
+        g_return_if_fail (_G_PASTE_IS_SETTINGS (self));                                                \
+        const GPasteSettingsPrivate *priv = _g_paste_settings_get_instance_private (self);             \
         g_settings_reset (priv->settings, G_PASTE_##key##_SETTING);                                    \
     }                                                                                                  \
     static void                                                                                        \
@@ -96,7 +92,7 @@ static guint signals[LAST_SIGNAL] = { 0 };
     g_paste_settings_set_##name (GPasteSettings *self,                                                 \
                                  type            value)                                                \
     {                                                                                                  \
-        g_return_if_fail (G_PASTE_IS_SETTINGS (self));                                                 \
+        g_return_if_fail (_G_PASTE_IS_SETTINGS (self));                                                \
         guards                                                                                         \
         GPasteSettingsPrivate *priv = g_paste_settings_get_instance_private (self);                    \
         clear_func                                                                                     \
@@ -108,7 +104,7 @@ static guint signals[LAST_SIGNAL] = { 0 };
     SETTING (name, key, type, setting_type, fail, {}, {},)
 
 #define BOOLEAN_SETTING(name, key) TRIVIAL_SETTING (name, key, gboolean, boolean, FALSE)
-#define UNSIGNED_SETTING(name, key) TRIVIAL_SETTING (name, key, guint32, uint, 0)
+#define UNSIGNED_SETTING(name, key) TRIVIAL_SETTING (name, key, guint64, uint64, 0)
 
 #define STRING_SETTING(name, key) SETTING (name, key, const gchar *, string, NULL,                \
                                            g_return_if_fail (value);                              \
@@ -130,6 +126,29 @@ static guint signals[LAST_SIGNAL] = { 0 };
 #define NEW_SIGNAL_DETAILED(name, arg_type) NEW_SIGNAL_FULL (name, G_SIGNAL_RUN_LAST | G_SIGNAL_DETAILED, arg_type, arg_type)
 #define NEW_SIGNAL_DETAILED_STATIC(name, arg_type) NEW_SIGNAL_FULL (name, G_SIGNAL_RUN_LAST | G_SIGNAL_DETAILED, arg_type, arg_type | G_SIGNAL_TYPE_STATIC_SCOPE)
 
+/**
+ * g_paste_settings_get_close_on_select:
+ * @self: a #GPasteSettings instance
+ *
+ * Get the "close-on-select" setting
+ *
+ * Returns: the value of the "close-on-select" setting
+ */
+/**
+ * g_paste_settings_reset_close_on_select:
+ * @self: a #GPasteSettings instance
+ *
+ * Reset the "close-on-select" setting
+ */
+/**
+ * g_paste_settings_set_close_on_select:
+ * @self: a #GPasteSettings instance
+ * @value: the new history name
+ *
+ * Change the "close-on-select" setting
+ */
+BOOLEAN_SETTING (close_on_select, CLOSE_ON_SELECT)
+
 /**
  * g_paste_settings_get_element_size:
  * @self: a #GPasteSettings instance
@@ -143,8 +162,6 @@ static guint signals[LAST_SIGNAL] = { 0 };
  * @self: a #GPasteSettings instance
  *
  * Reset the "element-size" setting
- *
- * Returns:
  */
 /**
  * g_paste_settings_set_element_size:
@@ -152,8 +169,6 @@ static guint signals[LAST_SIGNAL] = { 0 };
  * @value: the maximum displayed size of an item
  *
  * Change the "element-size" setting
- *
- * Returns:
  */
 UNSIGNED_SETTING (element_size, ELEMENT_SIZE)
 
@@ -170,8 +185,6 @@ UNSIGNED_SETTING (element_size, ELEMENT_SIZE)
  * @self: a #GPasteSettings instance
  *
  * Reset the "growing-lines" setting
- *
- * Returns:
  */
 /**
  * g_paste_settings_set_growing_lines:
@@ -179,8 +192,6 @@ UNSIGNED_SETTING (element_size, ELEMENT_SIZE)
  * @value: whether to detect or not growing lines
  *
  * Change the "growing-lines" setting
- *
- * Returns:
  */
 BOOLEAN_SETTING (growing_lines, GROWING_LINES)
 
@@ -197,8 +208,6 @@ BOOLEAN_SETTING (growing_lines, GROWING_LINES)
  * @self: a #GPasteSettings instance
  *
  * Reset the "history-name" setting
- *
- * Returns:
  */
 /**
  * g_paste_settings_set_history_name:
@@ -206,8 +215,6 @@ BOOLEAN_SETTING (growing_lines, GROWING_LINES)
  * @value: the new history name
  *
  * Change the "history-name" setting
- *
- * Returns:
  */
 STRING_SETTING (history_name, HISTORY_NAME)
 
@@ -224,8 +231,6 @@ STRING_SETTING (history_name, HISTORY_NAME)
  * @self: a #GPasteSettings instance
  *
  * Reset the "images-support" setting
- *
- * Returns:
  */
 /**
  * g_paste_settings_set_images_support:
@@ -233,8 +238,6 @@ STRING_SETTING (history_name, HISTORY_NAME)
  * @value: the new history name
  *
  * Change the "images-support" setting
- *
- * Returns:
  */
 BOOLEAN_SETTING (images_support, IMAGES_SUPPORT)
 
@@ -251,8 +254,6 @@ BOOLEAN_SETTING (images_support, IMAGES_SUPPORT)
  * @self: a #GPasteSettings instance
  *
  * Reset the "launch-ui" setting
- *
- * Returns:
  */
 /**
  * g_paste_settings_set_launch_ui:
@@ -260,8 +261,6 @@ BOOLEAN_SETTING (images_support, IMAGES_SUPPORT)
  * @value: the new keyboard shortcut
  *
  * Change the "launch-ui" setting
- *
- * Returns:
  */
 STRING_SETTING (launch_ui, LAUNCH_UI)
 
@@ -278,8 +277,6 @@ STRING_SETTING (launch_ui, LAUNCH_UI)
  * @self: a #GPasteSettings instance
  *
  * Reset the "make-password" setting
- *
- * Returns:
  */
 /**
  * g_paste_settings_set_make_password:
@@ -287,8 +284,6 @@ STRING_SETTING (launch_ui, LAUNCH_UI)
  * @value: the new keyboard shortcut
  *
  * Change the "make-password" setting
- *
- * Returns:
  */
 STRING_SETTING (make_password, MAKE_PASSWORD)
 
@@ -305,8 +300,6 @@ STRING_SETTING (make_password, MAKE_PASSWORD)
  * @self: a #GPasteSettings instance
  *
  * Reset the "max-displayed-history-size" setting
- *
- * Returns:
  */
 /**
  * g_paste_settings_set_max_displayed_history_size:
@@ -314,8 +307,6 @@ STRING_SETTING (make_password, MAKE_PASSWORD)
  * @value: the maximum number of items to display
  *
  * Change the "max-displayed-history-size" setting
- *
- * Returns:
  */
 UNSIGNED_SETTING (max_displayed_history_size, MAX_DISPLAYED_HISTORY_SIZE)
 
@@ -332,8 +323,6 @@ UNSIGNED_SETTING (max_displayed_history_size, MAX_DISPLAYED_HISTORY_SIZE)
  * @self: a #GPasteSettings instance
  *
  * Reset the "max-history-size" setting
- *
- * Returns:
  */
 /**
  * g_paste_settings_set_max_history_size:
@@ -341,8 +330,6 @@ UNSIGNED_SETTING (max_displayed_history_size, MAX_DISPLAYED_HISTORY_SIZE)
  * @value: the maximum number of items the history can contain
  *
  * Change the "max-history-size" setting
- *
- * Returns:
  */
 UNSIGNED_SETTING (max_history_size, MAX_HISTORY_SIZE)
 
@@ -359,8 +346,6 @@ UNSIGNED_SETTING (max_history_size, MAX_HISTORY_SIZE)
  * @self: a #GPasteSettings instance
  *
  * Reset the "max-memory-usage" setting
- *
- * Returns:
  */
 /**
  * g_paste_settings_set_max_memory_usage:
@@ -368,8 +353,6 @@ UNSIGNED_SETTING (max_history_size, MAX_HISTORY_SIZE)
  * @value: the maximum amout of memory we can use
  *
  * Change the "max-memory-usage" setting
- *
- * Returns:
  */
 UNSIGNED_SETTING (max_memory_usage, MAX_MEMORY_USAGE)
 
@@ -386,8 +369,6 @@ UNSIGNED_SETTING (max_memory_usage, MAX_MEMORY_USAGE)
  * @self: a #GPasteSettings instance
  *
  * Reset the "max-text-item-size" setting
- *
- * Returns:
  */
 /**
  * g_paste_settings_set_max_text_item_size:
@@ -395,8 +376,6 @@ UNSIGNED_SETTING (max_memory_usage, MAX_MEMORY_USAGE)
  * @value: the maximum size for a textual item to be handled
  *
  * Change the "max-text-item-size" setting
- *
- * Returns:
  */
 UNSIGNED_SETTING (max_text_item_size, MAX_TEXT_ITEM_SIZE)
 
@@ -413,8 +392,6 @@ UNSIGNED_SETTING (max_text_item_size, MAX_TEXT_ITEM_SIZE)
  * @self: a #GPasteSettings instance
  *
  * Reset the "min-text-item-size" setting
- *
- * Returns:
  */
 /**
  * g_paste_settings_set_min_text_item_size:
@@ -422,8 +399,6 @@ UNSIGNED_SETTING (max_text_item_size, MAX_TEXT_ITEM_SIZE)
  * @value: the minimum size for a textual item to be handled
  *
  * Change the "min-text-item-size" setting
- *
- * Returns:
  */
 UNSIGNED_SETTING (min_text_item_size, MIN_TEXT_ITEM_SIZE)
 
@@ -440,8 +415,6 @@ UNSIGNED_SETTING (min_text_item_size, MIN_TEXT_ITEM_SIZE)
  * @self: a #GPasteSettings instance
  *
  * Reset the "pop" setting
- *
- * Returns:
  */
 /**
  * g_paste_settings_set_pop:
@@ -449,8 +422,6 @@ UNSIGNED_SETTING (min_text_item_size, MIN_TEXT_ITEM_SIZE)
  * @value: the new keyboard shortcut
  *
  * Change the "pop" setting
- *
- * Returns:
  */
 STRING_SETTING (pop, POP)
 
@@ -467,8 +438,6 @@ STRING_SETTING (pop, POP)
  * @self: a #GPasteSettings instance
  *
  * Reset the "primary-to-history" setting
- *
- * Returns:
  */
 /**
  * g_paste_settings_set_primary_to_history:
@@ -476,11 +445,32 @@ STRING_SETTING (pop, POP)
  * @value: whether to track or not the primary selection changes as clipboard ones
  *
  * Change the "primary-to-history" setting
- *
- * Returns:
  */
 BOOLEAN_SETTING (primary_to_history, PRIMARY_TO_HISTORY)
 
+/**
+ * g_paste_settings_get_rich_text_support:
+ * @self: a #GPasteSettings instance
+ *
+ * Get the "rich-text-support" setting
+ *
+ * Returns: the value of the "rich-text-support" setting
+ */
+/**
+ * g_paste_settings_reset_rich_text_support:
+ * @self: a #GPasteSettings instance
+ *
+ * Reset the "rich-text-support" setting
+ */
+/**
+ * g_paste_settings_set_rich_text_support:
+ * @self: a #GPasteSettings instance
+ * @value: the new history name
+ *
+ * Change the "rich-text-support" setting
+ */
+BOOLEAN_SETTING (rich_text_support, RICH_TEXT_SUPPORT)
+
 /**
  * g_paste_settings_get_save_history:
  * @self: a #GPasteSettings instance
@@ -494,8 +484,6 @@ BOOLEAN_SETTING (primary_to_history, PRIMARY_TO_HISTORY)
  * @self: a #GPasteSettings instance
  *
  * Reset the "save-history" setting
- *
- * Returns:
  */
 /**
  * g_paste_settings_set_save_history:
@@ -503,8 +491,6 @@ BOOLEAN_SETTING (primary_to_history, PRIMARY_TO_HISTORY)
  * @value: whether to save or not the history
  *
  * Change the "save-history" setting
- *
- * Returns:
  */
 BOOLEAN_SETTING (save_history, SAVE_HISTORY)
 
@@ -521,8 +507,6 @@ BOOLEAN_SETTING (save_history, SAVE_HISTORY)
  * @self: a #GPasteSettings instance
  *
  * Reset the "show-history" setting
- *
- * Returns:
  */
 /**
  * g_paste_settings_set_show_history:
@@ -530,8 +514,6 @@ BOOLEAN_SETTING (save_history, SAVE_HISTORY)
  * @value: the new keyboard shortcut
  *
  * Change the "show-history" setting
- *
- * Returns:
  */
 STRING_SETTING (show_history, SHOW_HISTORY)
 
@@ -548,8 +530,6 @@ STRING_SETTING (show_history, SHOW_HISTORY)
  * @self: a #GPasteSettings instance
  *
  * Reset the "sync-clipboard-to-primary" setting
- *
- * Returns:
  */
 /**
  * g_paste_settings_set_sync_clipboard_to_primary:
@@ -557,8 +537,6 @@ STRING_SETTING (show_history, SHOW_HISTORY)
  * @value: the new keyboard shortcut
  *
  * Change the "sync-clipboard-to-primary" setting
- *
- * Returns:
  */
 STRING_SETTING (sync_clipboard_to_primary, SYNC_CLIPBOARD_TO_PRIMARY)
 
@@ -575,8 +553,6 @@ STRING_SETTING (sync_clipboard_to_primary, SYNC_CLIPBOARD_TO_PRIMARY)
  * @self: a #GPasteSettings instance
  *
  * Reset the "sync-primary-to-clipboard" setting
- *
- * Returns:
  */
 /**
  * g_paste_settings_set_sync_primary_to_clipboard:
@@ -584,8 +560,6 @@ STRING_SETTING (sync_clipboard_to_primary, SYNC_CLIPBOARD_TO_PRIMARY)
  * @value: the new keyboard shortcut
  *
  * Change the "sync-primary-to-clipboard" setting
- *
- * Returns:
  */
 STRING_SETTING (sync_primary_to_clipboard, SYNC_PRIMARY_TO_CLIPBOARD)
 
@@ -602,8 +576,6 @@ STRING_SETTING (sync_primary_to_clipboard, SYNC_PRIMARY_TO_CLIPBOARD)
  * @self: a #GPasteSettings instance
  *
  * Reset the "synchronize-clipboards" setting
- *
- * Returns:
  */
 /**
  * g_paste_settings_set_synchronize_clipboards:
@@ -611,8 +583,6 @@ STRING_SETTING (sync_primary_to_clipboard, SYNC_PRIMARY_TO_CLIPBOARD)
  * @value: whether to synchronize the clipboard and the primary selection or not
  *
  * Change the "synchronize-clipboards" setting
- *
- * Returns:
  */
 BOOLEAN_SETTING (synchronize_clipboards, SYNCHRONIZE_CLIPBOARDS)
 
@@ -629,8 +599,6 @@ BOOLEAN_SETTING (synchronize_clipboards, SYNCHRONIZE_CLIPBOARDS)
  * @self: a #GPasteSettings instance
  *
  * Reset the "track-changes" setting
- *
- * Returns:
  */
 /**
  * g_paste_settings_set_track_changes:
@@ -638,8 +606,6 @@ BOOLEAN_SETTING (synchronize_clipboards, SYNCHRONIZE_CLIPBOARDS)
  * @value: whether to track or not the clipboard changes
  *
  * Change the "track-changes" setting
- *
- * Returns:
  */
 BOOLEAN_SETTING (track_changes, TRACK_CHANGES)
 
@@ -656,8 +622,6 @@ BOOLEAN_SETTING (track_changes, TRACK_CHANGES)
  * @self: a #GPasteSettings instance
  *
  * Reset the "track-extension-state" setting
- *
- * Returns:
  */
 /**
  * g_paste_settings_set_track_extension_state:
@@ -665,8 +629,6 @@ BOOLEAN_SETTING (track_changes, TRACK_CHANGES)
  * @value: whether to stop tracking or not the clipboard changes when an applet exits
  *
  * Change the "track-extension-state" setting
- *
- * Returns:
  */
 BOOLEAN_SETTING (track_extension_state, TRACK_EXTENSION_STATE)
 
@@ -683,8 +645,6 @@ BOOLEAN_SETTING (track_extension_state, TRACK_EXTENSION_STATE)
  * @self: a #GPasteSettings instance
  *
  * Reset the "trim-items" setting
- *
- * Returns:
  */
 /**
  * g_paste_settings_set_trim_items:
@@ -692,8 +652,6 @@ BOOLEAN_SETTING (track_extension_state, TRACK_EXTENSION_STATE)
  * @value: whether to trim or not textual items
  *
  * Change the "trim-items" setting
- *
- * Returns:
  */
 BOOLEAN_SETTING (trim_items, TRIM_ITEMS)
 
@@ -710,8 +668,6 @@ BOOLEAN_SETTING (trim_items, TRIM_ITEMS)
  * @self: a #GPasteSettings instance
  *
  * Reset the "upload" setting
- *
- * Returns:
  */
 /**
  * g_paste_settings_set_upload:
@@ -719,13 +675,9 @@ BOOLEAN_SETTING (trim_items, TRIM_ITEMS)
  * @value: the new keyboard shortcut
  *
  * Change the "upload" setting
- *
- * Returns:
  */
 STRING_SETTING (upload, UPLOAD)
 
-#ifdef ENABLE_EXTENSION
-#define EXTENSION_NAME "GPaste@gnome-shell-extensions.gnome.org"
 /**
  * g_paste_settings_get_extension_enabled:
  * @self: a #GPasteSettings instance
@@ -737,8 +689,8 @@ STRING_SETTING (upload, UPLOAD)
 G_PASTE_VISIBLE gboolean
 g_paste_settings_get_extension_enabled (const GPasteSettings *self)
 {
-    g_return_val_if_fail (G_PASTE_IS_SETTINGS ((gpointer) self), FALSE);
-    GPasteSettingsPrivate *priv = g_paste_settings_get_instance_private (self);
+    g_return_val_if_fail (_G_PASTE_IS_SETTINGS ((gpointer) self), FALSE);
+    const GPasteSettingsPrivate *priv = _g_paste_settings_get_instance_private (self);
     return priv->extension_enabled;
 }
 
@@ -754,7 +706,7 @@ g_paste_settings_private_set_extension_enabled_from_dconf (GPasteSettingsPrivate
     g_auto (GStrv) extensions = g_paste_settings_private_get_enabled_extensions (priv);
     for (GStrv e = extensions; *e; ++e)
     {
-        if (!g_strcmp0 (*e, EXTENSION_NAME))
+        if (g_paste_str_equal (*e, G_PASTE_EXTENSION_NAME))
         {
             priv->extension_enabled = TRUE;
             return;
@@ -769,33 +721,33 @@ g_paste_settings_private_set_extension_enabled_from_dconf (GPasteSettingsPrivate
  * @value: whether to enable or not the gnome-shell extension
  *
  * Change the "extension-enabled" special setting
- *
- * Returns:
  */
 G_PASTE_VISIBLE void
 g_paste_settings_set_extension_enabled (GPasteSettings *self,
                                         gboolean        value)
 {
-    g_return_if_fail (G_PASTE_IS_SETTINGS (self));
+    g_return_if_fail (_G_PASTE_IS_SETTINGS (self));
+
     GPasteSettingsPrivate *priv = g_paste_settings_get_instance_private (self);
     g_auto (GStrv) extensions = NULL;
+
     if (!priv->shell_settings || (value == priv->extension_enabled))
         return;
 
     extensions = g_paste_settings_private_get_enabled_extensions (priv);
-    gsize nb = g_strv_length (extensions);
+    guint64 nb = g_strv_length (extensions);
     if (value)
     {
         extensions = g_realloc (extensions, (nb + 2) * sizeof (gchar *));
-        extensions[nb] = g_strdup (EXTENSION_NAME);
+        extensions[nb] = g_strdup (G_PASTE_EXTENSION_NAME);
         extensions[nb+1] = NULL;
     }
     else
     {
         gboolean found = FALSE;
-        for (gsize i = 0; i < nb; ++i)
+        for (guint64 i = 0; i < nb; ++i)
         {
-            if (!found && !g_strcmp0 (extensions[i], EXTENSION_NAME))
+            if (!found && g_paste_str_equal (extensions[i], G_PASTE_EXTENSION_NAME))
             {
                 found = TRUE;
                 g_free (extensions[i]);
@@ -826,7 +778,6 @@ g_paste_settings_shell_settings_changed (GSettings   *settings G_GNUC_UNUSED,
                    G_PASTE_EXTENSION_ENABLED_SETTING,
                    NULL);
 }
-#endif
 
 static void
 g_paste_settings_rebind (GPasteSettings *self,
@@ -846,61 +797,65 @@ g_paste_settings_settings_changed (GSettings   *settings G_GNUC_UNUSED,
     GPasteSettings *self = G_PASTE_SETTINGS (user_data);
     GPasteSettingsPrivate *priv = g_paste_settings_get_instance_private (self);
 
-    if (!g_strcmp0 (key, G_PASTE_ELEMENT_SIZE_SETTING))
+    if (g_paste_str_equal (key, G_PASTE_CLOSE_ON_SELECT_SETTING))
+        g_paste_settings_private_set_close_on_select_from_dconf (priv);
+    else if (g_paste_str_equal (key, G_PASTE_ELEMENT_SIZE_SETTING))
         g_paste_settings_private_set_element_size_from_dconf (priv);
-    else if (!g_strcmp0 (key, G_PASTE_GROWING_LINES_SETTING))
+    else if (g_paste_str_equal (key, G_PASTE_GROWING_LINES_SETTING))
         g_paste_settings_private_set_growing_lines_from_dconf (priv);
-    else if (!g_strcmp0 (key, G_PASTE_HISTORY_NAME_SETTING))
+    else if (g_paste_str_equal (key, G_PASTE_HISTORY_NAME_SETTING))
         g_paste_settings_private_set_history_name_from_dconf (priv);
-    else if (!g_strcmp0 (key, G_PASTE_IMAGES_SUPPORT_SETTING))
+    else if (g_paste_str_equal (key, G_PASTE_IMAGES_SUPPORT_SETTING))
         g_paste_settings_private_set_images_support_from_dconf (priv);
-    else if (!g_strcmp0 (key, G_PASTE_LAUNCH_UI_SETTING))
+    else if (g_paste_str_equal (key, G_PASTE_LAUNCH_UI_SETTING))
     {
         g_paste_settings_private_set_launch_ui_from_dconf (priv);
         g_paste_settings_rebind (self, G_PASTE_LAUNCH_UI_SETTING);
     }
-    else if (!g_strcmp0 (key, G_PASTE_MAKE_PASSWORD_SETTING))
+    else if (g_paste_str_equal (key, G_PASTE_MAKE_PASSWORD_SETTING))
     {
         g_paste_settings_private_set_make_password_from_dconf (priv);
         g_paste_settings_rebind (self, G_PASTE_MAKE_PASSWORD_SETTING);
     }
-    else if (!g_strcmp0 (key, G_PASTE_MAX_DISPLAYED_HISTORY_SIZE_SETTING))
+    else if (g_paste_str_equal (key, G_PASTE_MAX_DISPLAYED_HISTORY_SIZE_SETTING))
         g_paste_settings_private_set_max_displayed_history_size_from_dconf (priv);
-    else if (!g_strcmp0 (key, G_PASTE_MAX_HISTORY_SIZE_SETTING))
+    else if (g_paste_str_equal (key, G_PASTE_MAX_HISTORY_SIZE_SETTING))
         g_paste_settings_private_set_max_history_size_from_dconf (priv);
-    else if (!g_strcmp0 (key, G_PASTE_MAX_MEMORY_USAGE_SETTING))
+    else if (g_paste_str_equal (key, G_PASTE_MAX_MEMORY_USAGE_SETTING))
         g_paste_settings_private_set_max_memory_usage_from_dconf (priv);
-    else if (!g_strcmp0 (key, G_PASTE_MAX_TEXT_ITEM_SIZE_SETTING))
+    else if (g_paste_str_equal (key, G_PASTE_MAX_TEXT_ITEM_SIZE_SETTING))
         g_paste_settings_private_set_max_text_item_size_from_dconf (priv);
-    else if (!g_strcmp0 (key, G_PASTE_MIN_TEXT_ITEM_SIZE_SETTING))
+    else if (g_paste_str_equal (key, G_PASTE_MIN_TEXT_ITEM_SIZE_SETTING))
         g_paste_settings_private_set_min_text_item_size_from_dconf (priv);
-    else if (!g_strcmp0 (key, G_PASTE_POP_SETTING))
+    else if (g_paste_str_equal (key, G_PASTE_POP_SETTING))
     {
         g_paste_settings_private_set_pop_from_dconf (priv);
         g_paste_settings_rebind (self, G_PASTE_POP_SETTING);
     }
-    else if (!g_strcmp0 (key, G_PASTE_PRIMARY_TO_HISTORY_SETTING ))
+    else if (g_paste_str_equal (key, G_PASTE_PRIMARY_TO_HISTORY_SETTING ))
         g_paste_settings_private_set_primary_to_history_from_dconf (priv);
-    else if (!g_strcmp0 (key, G_PASTE_SAVE_HISTORY_SETTING))
+    else if (g_paste_str_equal (key, G_PASTE_RICH_TEXT_SUPPORT_SETTING))
+        g_paste_settings_private_set_rich_text_support_from_dconf (priv);
+    else if (g_paste_str_equal (key, G_PASTE_SAVE_HISTORY_SETTING))
         g_paste_settings_private_set_save_history_from_dconf (priv);
-    else if (!g_strcmp0 (key, G_PASTE_SHOW_HISTORY_SETTING))
+    else if (g_paste_str_equal (key, G_PASTE_SHOW_HISTORY_SETTING))
     {
         g_paste_settings_private_set_show_history_from_dconf (priv);
         g_paste_settings_rebind (self, G_PASTE_SHOW_HISTORY_SETTING);
     }
-    else if (!g_strcmp0 (key, G_PASTE_SYNC_CLIPBOARD_TO_PRIMARY_SETTING))
+    else if (g_paste_str_equal (key, G_PASTE_SYNC_CLIPBOARD_TO_PRIMARY_SETTING))
     {
         g_paste_settings_private_set_sync_clipboard_to_primary_from_dconf (priv);
         g_paste_settings_rebind (self, G_PASTE_SYNC_CLIPBOARD_TO_PRIMARY_SETTING);
     }
-    else if (!g_strcmp0 (key, G_PASTE_SYNC_PRIMARY_TO_CLIPBOARD_SETTING))
+    else if (g_paste_str_equal (key, G_PASTE_SYNC_PRIMARY_TO_CLIPBOARD_SETTING))
     {
         g_paste_settings_private_set_sync_primary_to_clipboard_from_dconf (priv);
         g_paste_settings_rebind (self, G_PASTE_SYNC_PRIMARY_TO_CLIPBOARD_SETTING);
     }
-    else if (!g_strcmp0 (key, G_PASTE_SYNCHRONIZE_CLIPBOARDS_SETTING))
+    else if (g_paste_str_equal (key, G_PASTE_SYNCHRONIZE_CLIPBOARDS_SETTING))
         g_paste_settings_private_set_synchronize_clipboards_from_dconf (priv);
-    else if (!g_strcmp0 (key, G_PASTE_TRACK_CHANGES_SETTING))
+    else if (g_paste_str_equal (key, G_PASTE_TRACK_CHANGES_SETTING))
     {
         g_paste_settings_private_set_track_changes_from_dconf (priv);
         g_signal_emit (self,
@@ -909,11 +864,11 @@ g_paste_settings_settings_changed (GSettings   *settings G_GNUC_UNUSED,
                        priv->track_changes,
                        NULL);
     }
-    else if (!g_strcmp0 (key, G_PASTE_TRACK_EXTENSION_STATE_SETTING))
+    else if (g_paste_str_equal (key, G_PASTE_TRACK_EXTENSION_STATE_SETTING))
         g_paste_settings_private_set_track_extension_state_from_dconf (priv);
-    else if (!g_strcmp0 (key, G_PASTE_TRIM_ITEMS_SETTING))
+    else if (g_paste_str_equal (key, G_PASTE_TRIM_ITEMS_SETTING))
         g_paste_settings_private_set_trim_items_from_dconf (priv);
-    else if (!g_strcmp0 (key, G_PASTE_UPLOAD_SETTING))
+    else if (g_paste_str_equal (key, G_PASTE_UPLOAD_SETTING))
     {
         g_paste_settings_private_set_upload_from_dconf (priv);
         g_paste_settings_rebind (self, G_PASTE_UPLOAD_SETTING);
@@ -930,24 +885,21 @@ g_paste_settings_settings_changed (GSettings   *settings G_GNUC_UNUSED,
 static void
 g_paste_settings_dispose (GObject *object)
 {
-    GPasteSettingsPrivate *priv = g_paste_settings_get_instance_private (G_PASTE_SETTINGS (object));
+    const GPasteSettingsPrivate *priv = _g_paste_settings_get_instance_private (G_PASTE_SETTINGS (object));
     GSettings *settings = priv->settings;
+    GSettings *shell_settings = priv->shell_settings;
 
     if (settings)
     {
-        g_signal_handler_disconnect (settings, priv->changed_signal);
+        g_signal_handler_disconnect (settings, priv->c_signals[C_CHANGED]);
         g_clear_object (&priv->settings);
     }
 
-#ifdef ENABLE_EXTENSION
-    GSettings *shell_settings = priv->shell_settings;
-
     if (shell_settings)
     {
-        g_signal_handler_disconnect (shell_settings, priv->shell_changed_signal);
+        g_signal_handler_disconnect (shell_settings, priv->c_signals[C_SHELL_CHANGED]);
         g_clear_object (&priv->shell_settings);
     }
-#endif
 
     G_OBJECT_CLASS (g_paste_settings_parent_class)->dispose (object);
 }
@@ -955,7 +907,7 @@ g_paste_settings_dispose (GObject *object)
 static void
 g_paste_settings_finalize (GObject *object)
 {
-    GPasteSettingsPrivate *priv = g_paste_settings_get_instance_private (G_PASTE_SETTINGS (object));
+    const GPasteSettingsPrivate *priv = _g_paste_settings_get_instance_private (G_PASTE_SETTINGS (object));
 
     g_free (priv->history_name);
     g_free (priv->launch_ui);
@@ -977,8 +929,44 @@ g_paste_settings_class_init (GPasteSettingsClass *klass)
     object_class->dispose = g_paste_settings_dispose;
     object_class->finalize = g_paste_settings_finalize;
 
+    /**
+     * GPasteSettings::changed:
+     * @settings: the object on which the signal was emitted
+     * @key: the name of the key that changed
+     *
+     * The "changed" signal is emitted when a key has potentially changed.
+     * You should call one of the g_paste_settings_get() calls to check the new
+     * value.
+     *
+     * This signal supports detailed connections.  You can connect to the
+     * detailed signal "changed::x" in order to only receive callbacks
+     * when key "x" changes.
+     */
     signals[CHANGED] = NEW_SIGNAL_DETAILED_STATIC ("changed", STRING);
+
+    /**
+     * GPasteSettings::rebind:
+     * @settings: the object on which the signal was emitted
+     * @key: the name of the key that changed
+     *
+     * The "rebind" signal is emitted when a key has potentially changed.
+     * You should call one of the g_paste_settings_get() calls to check the new
+     * value.
+     *
+     * This signal supports detailed connections.  You can connect to the
+     * detailed signal "rebind::x" in order to only receive callbacks
+     * when key "x" changes.
+     */
     signals[REBIND]  = NEW_SIGNAL_DETAILED        ("rebind" , STRING);
+
+    /**
+     * GPasteSettings::track:
+     * @settings: the object on which the signal was emitted
+     * @tracking_state: whether we're now tracking or not
+     *
+     * The "track" signal is emitted when the daemon starts or stops tracking
+     * clipboard changes
+     */
     signals[TRACK]   = NEW_SIGNAL                 ("track"  , BOOLEAN);
 }
 
@@ -997,11 +985,12 @@ g_paste_settings_init (GPasteSettings *self)
     priv->sync_primary_to_clipboard = NULL;
     priv->upload = NULL;
 
-    priv->changed_signal = g_signal_connect (settings,
-                                             "changed",
-                                             G_CALLBACK (g_paste_settings_settings_changed),
-                                             self);
+    priv->c_signals[C_CHANGED] = g_signal_connect (settings,
+                                                   "changed",
+                                                   G_CALLBACK (g_paste_settings_settings_changed),
+                                                   self);
 
+    g_paste_settings_private_set_close_on_select_from_dconf (priv);
     g_paste_settings_private_set_element_size_from_dconf (priv);
     g_paste_settings_private_set_growing_lines_from_dconf (priv);
     g_paste_settings_private_set_history_name_from_dconf (priv);
@@ -1015,6 +1004,7 @@ g_paste_settings_init (GPasteSettings *self)
     g_paste_settings_private_set_min_text_item_size_from_dconf (priv);
     g_paste_settings_private_set_pop_from_dconf (priv);
     g_paste_settings_private_set_primary_to_history_from_dconf (priv);
+    g_paste_settings_private_set_rich_text_support_from_dconf (priv);
     g_paste_settings_private_set_save_history_from_dconf (priv);
     g_paste_settings_private_set_show_history_from_dconf (priv);
     g_paste_settings_private_set_sync_clipboard_to_primary_from_dconf (priv);
@@ -1028,25 +1018,17 @@ g_paste_settings_init (GPasteSettings *self)
     priv->shell_settings = NULL;
     priv->extension_enabled = FALSE;
 
-#ifdef ENABLE_EXTENSION
-    GSettingsSchemaSource *source = g_settings_schema_source_get_default ();
-    if (!source)
-        return;
+    if (g_paste_util_has_gnome_shell ())
+    {
+        priv->shell_settings = g_settings_new (G_PASTE_SHELL_SETTINGS_NAME);
 
-    g_autoptr (GSettingsSchema) schema = g_settings_schema_source_lookup (source, G_PASTE_SHELL_SETTINGS_NAME, TRUE);
+        priv->c_signals[C_SHELL_CHANGED] = g_signal_connect (priv->shell_settings,
+                                                             "changed::" G_PASTE_SHELL_ENABLED_EXTENSIONS_SETTING,
+                                                             G_CALLBACK (g_paste_settings_shell_settings_changed),
+                                                             self);
 
-    if (!schema)
-        return;
-
-    priv->shell_settings = g_settings_new (G_PASTE_SHELL_SETTINGS_NAME);
-
-    priv->shell_changed_signal = g_signal_connect (priv->shell_settings,
-                                                  "changed::" G_PASTE_SHELL_ENABLED_EXTENSIONS_SETTING,
-                                                  G_CALLBACK (g_paste_settings_shell_settings_changed),
-                                                  self);
-
-    g_paste_settings_private_set_extension_enabled_from_dconf (priv);
-#endif
+        g_paste_settings_private_set_extension_enabled_from_dconf (priv);
+    }
 }
 
 /**
